# test_WriteArdbHTML.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Test Writing a card set to an Ardb HTML file"""

import time
from sutekh.SutekhInfo import SutekhInfo
from sutekh.tests.TestCore import SutekhTest
from sutekh.tests.core.test_PhysicalCardSet import aCardSetNames, \
        get_phys_cards
from sutekh.core.SutekhObjects import PhysicalCardSet
from sutekh.io.WriteArdbHTML import  WriteArdbHTML, sHTMLStyle
import unittest

sExpected1 = """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
 "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html lang="en" xml:lang="en" xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <meta content="text/html; charset=&quot;us-ascii&quot;" http-equiv="content-type" />
    <style type="text/css">%s</style>
    <title>VTES deck : Test Set 1 by A test author</title>
  </head><body>
    <div id="info">
      <h1 id="nametitle">
        <span>Deck Name :</span>
        <span class="value" id="namevalue">Test Set 1</span>
      </h1><h2 id="authortitle">
        <span>Author : </span>
        <span class="value" id="authornamevalue">A test author</span>
      </h2><h2 id="description">
        <span>Description : </span>
      </h2><p>
        <span class="value" id="descriptionvalue">A test comment</span>
      </p>
    </div><div id="crypt">
      <h3 id="crypttitle">
        <span>Crypt</span>
        <span>[1 vampires] Capacity min : 4 max : 4 average : 4.00</span>
      </h3><div id="crypttable">
        <table summary="Crypt card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://monger.vekn.org/showvamp.html?NAME=Abebe">Abebe</a>
                </span>
              </td><td />
              <td>
                <span class="tablevalue">4</span>
              </td><td>
                <span class="tablevalue">nec obf thn</span>
              </td><td />
              <td>
                <span class="tablevalue">Samedi (group 4)</span>
              </td>
            </tr>
          </tbody>
        </table>
      </div>
    </div><div id="library">
      <h3 id="librarytitle">
        <span>Library</span>
        <span class="stats" id="librarystats">[4 cards]</span>
      </h3><div class="librarytable">
        <h4 class="librarytype">
          <span>Action</span>
          <span class="stats">[1]</span>
        </h4><table summary="Library card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://monger.vekn.org/showcard.html?NAME=Abbot">Abbot</a>
                </span>
              </td>
            </tr>
          </tbody>
        </table><h4 class="librarytype">
          <span>Equipment</span>
          <span class="stats">[2]</span>
        </h4><table summary="Library card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://monger.vekn.org/showcard.html?NAME=.44%%20Magnum">.44 Magnum</a>
                </span>
              </td>
            </tr><tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://monger.vekn.org/showcard.html?NAME=AK-47">AK-47</a>
                </span>
              </td>
            </tr>
          </tbody>
        </table><h4 class="librarytype">
          <span>Master</span>
          <span class="stats">[1]</span>
        </h4><table summary="Library card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://monger.vekn.org/showcard.html?NAME=Abombwe">Abombwe</a>
                </span>
              </td>
            </tr>
          </tbody>
        </table>
      </div>
    </div><div>
      <span class="generator">Crafted with : Sutekh [ %s ]. [ %s ]</span>
    </div>
  </body>
</html>""" % (sHTMLStyle, SutekhInfo.VERSION_STR,
        time.strftime('%Y-%m-%d', time.localtime()))

sExpected2 = """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
 "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html lang="en" xml:lang="en" xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <meta content="text/html; charset=&quot;us-ascii&quot;" http-equiv="content-type" />
    <style type="text/css">%s</style>
    <title>VTES deck : Test Set 1 by A test author</title>
  </head><body>
    <div id="info">
      <h1 id="nametitle">
        <span>Deck Name :</span>
        <span class="value" id="namevalue">Test Set 1</span>
      </h1><h2 id="authortitle">
        <span>Author : </span>
        <span class="value" id="authornamevalue">A test author</span>
      </h2><h2 id="description">
        <span>Description : </span>
      </h2><p>
        <span class="value" id="descriptionvalue">A test comment</span>
      </p>
    </div><div id="crypt">
      <h3 id="crypttitle">
        <span>Crypt</span>
        <span>[1 vampires] Capacity min : 4 max : 4 average : 4.00</span>
      </h3><div id="crypttable">
        <table summary="Crypt card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">Abebe</span>
              </td><td />
              <td>
                <span class="tablevalue">4</span>
              </td><td>
                <span class="tablevalue">nec obf thn</span>
              </td><td />
              <td>
                <span class="tablevalue">Samedi (group 4)</span>
              </td>
            </tr>
          </tbody>
        </table>
      </div>
    </div><div id="library">
      <h3 id="librarytitle">
        <span>Library</span>
        <span class="stats" id="librarystats">[4 cards]</span>
      </h3><div class="librarytable">
        <h4 class="librarytype">
          <span>Action</span>
          <span class="stats">[1]</span>
        </h4><table summary="Library card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">Abbot</span>
              </td>
            </tr>
          </tbody>
        </table><h4 class="librarytype">
          <span>Equipment</span>
          <span class="stats">[2]</span>
        </h4><table summary="Library card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">.44 Magnum</span>
              </td>
            </tr><tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">AK-47</span>
              </td>
            </tr>
          </tbody>
        </table><h4 class="librarytype">
          <span>Master</span>
          <span class="stats">[1]</span>
        </h4><table summary="Library card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">Abombwe</span>
              </td>
            </tr>
          </tbody>
        </table>
      </div>
    </div><div>
      <span class="generator">Crafted with : Sutekh [ %s ]. [ %s ]</span>
    </div>
  </body>
</html>""" % (sHTMLStyle, SutekhInfo.VERSION_STR,
        time.strftime('%Y-%m-%d', time.localtime()))

sExpected3 = """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
 "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html lang="en" xml:lang="en" xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <meta content="text/html; charset=&quot;us-ascii&quot;" http-equiv="content-type" />
    <style type="text/css">%s</style>
    <title>VTES deck : Test Set 1 by A test author</title>
  </head><body>
    <div id="info">
      <h1 id="nametitle">
        <span>Deck Name :</span>
        <span class="value" id="namevalue">Test Set 1</span>
      </h1><h2 id="authortitle">
        <span>Author : </span>
        <span class="value" id="authornamevalue">A test author</span>
      </h2><h2 id="description">
        <span>Description : </span>
      </h2><p>
        <span class="value" id="descriptionvalue">A test comment</span>
      </p>
    </div><div id="crypt">
      <h3 id="crypttitle">
        <span>Crypt</span>
        <span>[1 vampires] Capacity min : 4 max : 4 average : 4.00</span>
      </h3><div id="crypttable">
        <table summary="Crypt card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://www.secretlibrary.info/?crypt=Abebe">Abebe</a>
                </span>
              </td><td />
              <td>
                <span class="tablevalue">4</span>
              </td><td>
                <span class="tablevalue">nec obf thn</span>
              </td><td />
              <td>
                <span class="tablevalue">Samedi (group 4)</span>
              </td>
            </tr>
          </tbody>
        </table>
      </div>
    </div><div id="library">
      <h3 id="librarytitle">
        <span>Library</span>
        <span class="stats" id="librarystats">[4 cards]</span>
      </h3><div class="librarytable">
        <h4 class="librarytype">
          <span>Action</span>
          <span class="stats">[1]</span>
        </h4><table summary="Library card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://www.secretlibrary.info/?lib=Abbot">Abbot</a>
                </span>
              </td>
            </tr>
          </tbody>
        </table><h4 class="librarytype">
          <span>Equipment</span>
          <span class="stats">[2]</span>
        </h4><table summary="Library card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://www.secretlibrary.info/?lib=.44+Magnum">.44 Magnum</a>
                </span>
              </td>
            </tr><tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://www.secretlibrary.info/?lib=AK-47">AK-47</a>
                </span>
              </td>
            </tr>
          </tbody>
        </table><h4 class="librarytype">
          <span>Master</span>
          <span class="stats">[1]</span>
        </h4><table summary="Library card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://www.secretlibrary.info/?lib=Abombwe">Abombwe</a>
                </span>
              </td>
            </tr>
          </tbody>
        </table>
      </div>
    </div><div>
      <span class="generator">Crafted with : Sutekh [ %s ]. [ %s ]</span>
    </div>
  </body>
</html>""" % (sHTMLStyle, SutekhInfo.VERSION_STR,
        time.strftime('%Y-%m-%d', time.localtime()))


class ARDBHTMLWriterTests(SutekhTest):
    """class for the ARDB HTML deck writer tests"""

    def test_deck_writer(self):
        """Test HTML deck writing"""
        # pylint: disable-msg=E1101, R0915, R0914
        # E1101: SQLObject + PyProtocols magic confuses pylint
        # R0915, R0914: Want a long, sequential test case to minimise
        # repeated setups, so it has lots of lines + variables
        aAddedPhysCards = get_phys_cards()
        # We have a physical card list, so create some physical card sets
        oPhysCardSet1 = PhysicalCardSet(name=aCardSetNames[0])
        oPhysCardSet1.comment = 'A test comment'
        oPhysCardSet1.author = 'A test author'

        for iLoop in range(5):
            oPhysCardSet1.addPhysicalCard(aAddedPhysCards[iLoop].id)
            oPhysCardSet1.syncUpdate()

        # Check output

        oWriter = WriteArdbHTML()
        sTempFileName =  self._create_tmp_file()
        oWriter.write(sTempFileName, oPhysCardSet1, oPhysCardSet1.cards, False)

        fIn = open(sTempFileName, 'rU')
        sData = fIn.read()
        fIn.close()

        self.assertEqual(sData, sExpected1)

        # Test other modes
        oWriter = WriteArdbHTML('Monger')
        sTempFileName =  self._create_tmp_file()
        oWriter.write(sTempFileName, oPhysCardSet1, oPhysCardSet1.cards, False)

        fIn = open(sTempFileName, 'rU')
        sData = fIn.read()
        fIn.close()

        self.assertEqual(sData, sExpected1)

        oWriter = WriteArdbHTML('None')
        sTempFileName =  self._create_tmp_file()
        oWriter.write(sTempFileName, oPhysCardSet1, oPhysCardSet1.cards, False)

        fIn = open(sTempFileName, 'rU')
        sData = fIn.read()
        fIn.close()

        self.assertEqual(sData, sExpected2)

        oWriter = WriteArdbHTML('Secret Library')
        sTempFileName =  self._create_tmp_file()
        oWriter.write(sTempFileName, oPhysCardSet1, oPhysCardSet1.cards, False)

        fIn = open(sTempFileName, 'rU')
        sData = fIn.read()
        fIn.close()

        self.assertEqual(sData, sExpected3)


if __name__ == "__main__":
    unittest.main()



# textile2html.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details

"""
Convert Sutekh textile documentation into HTML pages.
"""

import glob
import os
import textile

# pylint: disable-msg=C0103
# we ignore our usual naming conventions here

HTML_HEADER = """
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
        "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html lang="en" xml:lang="en" xmlns="http://www.w3.org/1999/xhtml">
<head>
    <title>%(title)s</title>
</head>
<body>
"""

HTML_FOOTER = """
</body>
</html>
"""

# pylint: enable-msg=C0103

def textile2html(sText, dContext):
    """Convert a Textile markup string to an HTML file."""
    sHtml = HTML_HEADER % dContext \
        + textile.textile(sText) \
        + HTML_FOOTER % dContext
    sHtml = sHtml.replace('<br />',' ') # remove pesky linebreaks
    return sHtml

def convert(sTextileDir, sHtmlDir):
    """Convert all .txt files in sTextileDir to .html files in sHtmlDir."""
    for sTextilePath in glob.glob(os.path.join(sTextileDir, "*.txt")):
        sBasename = os.path.basename(sTextilePath)
        # pylint: disable-msg=W0612
        # sExt is unused
        sFilename, sExt = os.path.splitext(sBasename)
        sHtmlPath = os.path.join(sHtmlDir, sFilename + ".html")

        # pylint: enable-msg=W0612

        dContext = {
            'title': "Sutekh " + sFilename
        }

        fTextile = file(sTextilePath, "rb")
        fHtml = file(sHtmlPath, "wb")

        fHtml.write(textile2html(fTextile.read(), dContext))

        fTextile.close()
        fHtml.close()

if __name__ == "__main__":
    convert("textile", "html")



# CardSetPrint.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2007 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details

"""Plugin for simple, direct printing of the card set."""

import gtk
import pango
from sutekh.core.SutekhObjects import PhysicalCardSet
from sutekh.gui.PluginManager import CardListPlugin
from sutekh.gui.SutekhDialog import do_complaint_error

try:
    # pylint: disable-msg=W0104
    # This is an existance check, so it does do nothing
    gtk.PrintOperation
except AttributeError:
    raise ImportError("GTK version does not contain PrintOperation."
            " Try PyGTK >= 2.10.")

class CardSetPrint(CardListPlugin):
    """Plugin for printing the card sets.

       Use gtk's Printing support to print out a simple list of the cards
       in the card set. This has less formatting than exporting via
       HTML, for instance, but does print directly.
       """
    dTableVersions = { PhysicalCardSet: [2, 3, 4, 5]}
    aModelsSupported = [PhysicalCardSet]

    # pylint: disable-msg=W0201
    # We don't care that we define _oSettings here, due to how plugin is called
    def get_menu_item(self):
        """Register on the 'Actions' Menu"""
        if not self.check_versions() or not self.check_model_type():
            return None

        # Keep print settings
        self._oSettings = None

        # internal state for printing
        self._aPageBreaks = None
        self._oPangoLayout = None

        oPrint = gtk.MenuItem("Print Card Set")
        oPrint.connect("activate", self.activate)
        return ('Actions', oPrint)

    # pylint: enable-msg=W0201

    # pylint: disable-msg=W0613
    # oWidget part of function signature
    def activate(self, oWidget):
        """In response to the menu choice, do the actual print operation."""
        oPrintOp = gtk.PrintOperation()

        if not self._oSettings is None:
            oPrintOp.set_print_settings(self._oSettings)

        oPrintOp.connect("begin-print", self.begin_print)
        oPrintOp.connect("end-print", self.end_print)
        oPrintOp.connect("draw-page", self.draw_page)

        oRes = oPrintOp.run(gtk.PRINT_OPERATION_ACTION_PRINT_DIALOG,
                self.parent)

        if oRes == gtk.PRINT_OPERATION_RESULT_ERROR:
            do_complaint_error("Error printing card set:\n")
        elif oRes == gtk.PRINT_OPERATION_RESULT_APPLY:
            self._oSettings = oPrintOp.get_print_settings()

    def begin_print(self, oPrintOp, oContext):
        """Set up printing context.

           This includes determining pagination and the number of pages.
           """
        oPrintOp.set_use_full_page(False)
        oPrintOp.set_unit(gtk.UNIT_MM)
        oPrintOp.set_n_pages(1) # safety net in case error occur later on

        fWidth, fHeight = oContext.get_width(), oContext.get_height()

        oLayout = oContext.create_pango_layout()
        oLayout.set_font_description(pango.FontDescription("sans 12"))
        oLayout.set_width(int(fWidth * pango.SCALE))

        oLayout.set_markup(self.cardlist_markup())

        iLineCount = oLayout.get_line_count()

        fPageHeight = 0
        aPageBreaks = []

        for iThisLine in range(iLineCount):
            oLine = oLayout.get_line(iThisLine)
            # pylint: disable-msg=W0612
            # not interested in oInkRect
            oInkRect, oLogicalRect = oLine.get_extents()

            # line height / SCALE
            fLineHeight = float(oLogicalRect[3]) / pango.SCALE

            if fPageHeight + fLineHeight > fHeight:
                aPageBreaks.append(iThisLine)
                fPageHeight = 0

            fPageHeight += fLineHeight

        oPrintOp.set_n_pages(len(aPageBreaks)+1)
        self._aPageBreaks = aPageBreaks
        self._oPangoLayout = oLayout

    # oPrintOp, oContext part of function signature
    def end_print(self, oPrintOp, oContext):
        """Clean up resources allocated in begin_print.
           """
        self._aPageBreaks = None
        self._oPangoLayout = None

    # oPrintOp part of function signature
    def draw_page(self, oPrintOp, oContext, iPageNum):
        """Page drawing callback.
           """
        # pylint: disable-msg=R0914
        # We use lots of variables for clarity
        iStartPageLine, iEndPageLine = 0, 0
        aPageBreaks = self._aPageBreaks
        oLayout = self._oPangoLayout

        if iPageNum > 0:
            iStartPageLine = aPageBreaks[iPageNum-1]

        if iPageNum < len(aPageBreaks):
            iEndPageLine = aPageBreaks[iPageNum]
        else:
            iEndPageLine = oLayout.get_line_count()

        oContext = oContext.get_cairo_context()
        oContext.set_source_rgb(0, 0, 0)
        oContext.set_line_width(0.1)

        oIter = oLayout.get_iter()

        fStartPos = 0.0
        iLine = 0

        while True:
            if iLine >= iStartPageLine:
                oLine = oIter.get_line()
                # pylint: disable-msg=W0612
                # not interested in oInkRect
                oInkRect, oLogicalRect = oLine.get_extents()

                fBaseLine = float(oIter.get_baseline()) / pango.SCALE

                if iLine == iStartPageLine:
                    fStartPos = fBaseLine

                # line x co-ordinate / SCALE
                fXPos = float(oLogicalRect[0]) / pango.SCALE
                # baseline - start pos - line y-co-ordinate / SCALE
                fYPos = fBaseLine - fStartPos - float(oLogicalRect[1]) / \
                        pango.SCALE

                oContext.move_to(fXPos, fYPos)
                oContext.layout_line_path(oLine)
                oContext.stroke_preserve()
                oContext.fill()

            iLine += 1
            if not (iLine < iEndPageLine and oIter.next_line()):
                break

    # pylint: enable-msg=W0613

    def cardlist_markup(self):
        """Format the card set nicely for printing."""
        def escape(sInput):
            """Escape strings so that markup and special characters
               don't break things."""
            from gobject import markup_escape_text
            if sInput:
                return markup_escape_text(sInput)
            else:
                return sInput # pass None straigh through

        oCS = self.get_card_set()

        aMarkup = []
        aMarkup.append("<u>%s</u>" % escape(oCS.name))
        aMarkup.append("  Author: %s" % escape(oCS.author))
        aMarkup.append("  Comment: %s" % escape(oCS.comment))
        aMarkup.append("  Annotations: %s" % escape(oCS.annotations))
        aMarkup.append("")

        oCardIter = self.model.get_card_iterator(None)
        # pylint: disable-msg=W0612
        # aAbsCards unused
        oGroupedIter, aAbsCards = self.model.grouped_card_iter(oCardIter)
        # pylint: enable-msg=W0612

        # Iterate over groups
        for sGroup, oGroupIter in oGroupedIter:
            # Check for null group
            if sGroup is None:
                sGroup = '<< None >>'

            aMarkup.append("<u>%s:</u>" % (escape(sGroup),))

            # Fill in Cards
            for oCard, oRow in oGroupIter:
                iCnt = oRow.get_card_count()
                aMarkup.append("  %ix %s" % (iCnt, escape(oCard.name)))

            aMarkup.append("")

        return "\n".join(aMarkup)

# pylint: disable-msg=C0103
# accept plugin name
plugin = CardSetPrint



# WriteArdbText.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details

"""Writer for ther ARDB text deck format.

   Example deck:

   Deck Name : Followers of Set Preconstructed Deck
   Author : L. Scott Johnson
   Description :
   Followers of Set Preconstructed Starter from Lords of the Night.

   http://www.white-wolf.com/vtes/index.php?line=Checklist_LordsOfTheNight

   Crypt [12 vampires] Capacity min: 2 max: 10 average: 5.84
   ------------------------------------------------------------
   2x Nakhthorheb			  10 OBF PRE SER           Follower :4
   ...

   Library [77 cards]
   ------------------------------------------------------------
   Action [20]
     2x Blithe Acceptance
     4x Dream World
   ...
   """

from sutekh.core.SutekhObjects import IAbstractCard
from sutekh.core.ArdbInfo import ArdbInfo

class WriteArdbText(ArdbInfo):
    """Create a string in ARDB's text format representing a dictionary
       of cards."""

    # pylint: disable-msg=R0201
    # method for consistency with the other methods
    def _gen_header(self, sSetName, sAuthor, sDescription):
        """Generate an ARDB text file header."""
        return "Deck Name : %s\n" \
               "Author : %s\n" \
               "Description :\n%s\n" % (sSetName, sAuthor, sDescription)
    # pylint: enable-msg=R0201

    def _gen_crypt(self, dCards):
        """Generaten an ARDB text file crypt description.

           dCards is mapping of (card id, card name) -> card count.
           """
        (dVamps, dCryptStats) = self._extract_crypt(dCards)

        sCrypt = "Crypt [%(size)d vampires] Capacity min: %(min)d " \
                "max: %(max)d average: %(avg).2f\n" \
                "-------------------------------------------------" \
                "-----------\n" \
                % dCryptStats

        for tKey in sorted(dVamps, key=lambda x: x[1]):
            sName = tKey[1]
            iCount = dVamps[tKey]

            # pylint: disable-msg=E1101
            # IAbstractCard confuses pylint
            oCard = IAbstractCard(sName)
            if len(oCard.creed) > 0:
                sClan = "Imbued"
                iCapacity = oCard.life
            else:
                sClan = [x.name for x in oCard.clan][0]
                iCapacity = oCard.capacity
            sDisciplines = self._gen_disciplines(oCard)

            sCrypt += "  %dx %s %d %s %s :%d\n" % \
                 (iCount, sName, iCapacity, sDisciplines, sClan,
                         oCard.group)

        return sCrypt

    def _gen_library(self, dCards):
        """Generaten an ARDB text file library description.

           dCards is mapping of (card id, card name) -> card count.
           """
        (dLib, iLibSize) = self._extract_library(dCards)

        sLib = "Library [%d cards]\n" \
            "------------------------------------------------------------\n" \
            % (iLibSize,)

        dTypes = {}
        for iId, sName, sTypeString, sSet in dLib:
            if sTypeString not in dTypes:
                dTypes[sTypeString] = {}
            dTypes[sTypeString].setdefault((iId, sName), 0)
            dTypes[sTypeString][(iId, sName)] += dLib[(iId, sName,
                sTypeString, sSet)]

        for sTypeString in sorted(dTypes):
            dCards = dTypes[sTypeString]
            iTotal = sum(dCards.values())

            sLib += "%s [%d]\n" % (sTypeString, iTotal)

            for (iId, sName), iCount in dCards.iteritems():
                sLib += "  %dx %s\n" % (iCount, sName)

            sLib += "\n"

        return sLib

    # pylint: disable-msg=R0913
    # we need all these arguments
    def write(self, fOut, sSetName, sAuthor, sDescription, oCardIter):
        """Takes filename, deck details and a dictionary of cards, of the
           form dCard[(id, name, set)] = count and writes the file."""
        dCards = self._get_cards(oCardIter)
        fOut.write(self._gen_header(sSetName, sAuthor, sDescription))
        fOut.write("\n")
        fOut.write(self._gen_crypt(dCards))
        fOut.write("\n")
        fOut.write(self._gen_library(dCards))

    # pylint: enable-msg=R0913



# ELDBUtilities.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Utilitity functions for importing from & exporting to FELDB"""

import unicodedata
from sutekh.core.SutekhObjects import AbstractCard

def type_of_card(oCard):
    """Return either Crypt or Library as required."""
    sType = list(oCard.cardtype)[0].name
    if sType == "Vampire" or sType == "Imbued":
        return "Crypt"
    else:
        return "Library"

def norm_name(oCard):
    """Transform a card name to the ELDB equivilant"""
    sName = oCard.name
    if oCard.level is not None:
        sName = sName.replace("(Advanced)", "(ADV)")
    if sName.startswith("The ") and sName != "The Kikiyaon":
        # Annoying ELDB special case
        sName = sName[4:] + ", The"
    if sName.startswith("An "):
        # FELDB does this as well
        sName = sName[3:] + ", An"
    sName = sName.replace("'", "`")
    return unicodedata.normalize('NFKD', sName).encode('ascii','ignore')

def gen_name_lookups():
    """Create a lookup table to map ELDB names to Sutekh names -
       reduces the number of user queries"""
    dNameCache = {}
    for oCard in AbstractCard.select():
        sSutekhName = oCard.name
        sELDBName = norm_name(oCard)
        if sELDBName != sSutekhName:
            # Since we will need to check wether a card is in the dictionary
            # anyway (missed cases, etc), there's no point in having the
            # identity entries
            dNameCache[sELDBName] = sSutekhName
    return dNameCache



# Rulebook.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Simon Cross <hodgestar+sutekh@gmail.com>
# GPL - see COPYING for details

"""Downloads rulebook HTML pages and makes them available via the Help menu."""

import gtk
import os
import re
import webbrowser
import urllib
from sutekh.gui.PluginManager import CardListPlugin
from sutekh.gui.ProgressDialog import ProgressDialog
from sutekh.gui.SutekhFileWidget import SutekhFileButton
from sutekh.gui.SutekhDialog import SutekhDialog, do_complaint_error
from sutekh.SutekhUtility import prefs_dir, ensure_dir_exists
from sutekh.io.WwFile import WwFile

# pylint: disable-msg=R0904
# R0904 - gtk Widget, so has many public methods
class RulebookConfigDialog(SutekhDialog):
    """Dialog for configuring the Rulebook plugin."""

    POSSIBLE_FILES = [
            "Rulebook", "Imbued Rules", "Rulings", "V:EKN Tournament Rules",
            "VTES FAQ", "V:TES Complete Rules Reference",
    ]

    WW_RULEBOOK_URLS = {
        "Rulebook": "http://www.white-wolf.com/vtes/rulebook/rulebook.html",
        "Imbued Rules":
           "http://www.white-wolf.com/vtes/?line=Checklist_NightsOfReckoning",
        "Rulings": "http://www.white-wolf.com/vtes/index.php?line=rulings",
        "V:EKN Tournament Rules":
           "http://www.white-wolf.com/vtes/index.php?line=veknRules",
        "VTES FAQ": "http://www.thelasombra.com/vtes_faq.htm",
        "V:TES Complete Rules Reference":
           "http://www.white-wolf.com/vtes/?line=outline",
    }

    def __init__(self, oParent, bFirstTime=False):
        super(RulebookConfigDialog, self).__init__('Configure Rulebook Plugin',
                oParent, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                (gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL,
                    gtk.RESPONSE_CANCEL))

        oDescLabel = gtk.Label()
        if not bFirstTime:
            oDescLabel.set_markup('<b>Choose how to configure the rulebook '
                    'plugin</b>')
        else:
            oDescLabel.set_markup('<b>Choose how to configure the rulebook '
                    'plugin</b>\nChoose cancel to skip configuring the '
                    'plugin\nYou will not be prompted again')

        self.oSourceWw = gtk.RadioButton(
                label='Download from www.white-wolf.com and'
                ' www.thelasombra.com')

        self.oSourceUrl = gtk.RadioButton(self.oSourceWw,
                label='Download from other URL(s)')

        # SourceUrl sub-components
        oUrlSub = gtk.VBox(False, 2)
        self.dUrlSubEntries = {}
        for sName in self.POSSIBLE_FILES:
            oLabel = gtk.Label(sName)
            oEntry = gtk.Entry()
            oHBox = gtk.HBox(False, 2)
            oHBox.pack_start(oLabel, expand=False, padding=2)
            oHBox.pack_start(oEntry)
            oUrlSub.pack_start(oHBox)
            self.dUrlSubEntries[sName] = oEntry

        self.oSourceFile = gtk.RadioButton(self.oSourceWw,
                label='Select local files')

        # SourceFile sub-components
        oFileSub = gtk.VBox(False, 2)
        self.dFileSubButtons = {}
        for sName in self.POSSIBLE_FILES:
            oLabel = gtk.Label(sName)
            oFileButton = SutekhFileButton(oParent,
                "Select %s HTML file" % (sName,))
            oFileButton.add_filter_with_pattern('HTML files',
                ['*.html', '*.htm'])
            oHBox = gtk.HBox(False, 2)
            oHBox.pack_start(oLabel, expand=False, padding=2)
            oHBox.pack_start(oFileButton)
            oFileSub.pack_start(oHBox)
            self.dFileSubButtons[sName] = oFileButton

        # box for holding additional widgets for the various source
        # options
        oSubBox = gtk.VBox(False, 2)

        self.oSourceWw.connect('toggled', self._radio_toggled, oSubBox, None)
        self.oSourceUrl.connect('toggled', self._radio_toggled, oSubBox,
            oUrlSub)
        self.oSourceFile.connect('toggled', self._radio_toggled, oSubBox,
            oFileSub)

        self.oSourceWw.set_active(True)

        # pylint: disable-msg=E1101
        # pylint doesn't pick up vbox methods correctly
        self.vbox.pack_start(oDescLabel, False, False)
        self.vbox.pack_start(self.oSourceWw, False, False)
        self.vbox.pack_start(self.oSourceUrl, False, False)
        self.vbox.pack_start(self.oSourceFile, False, False)
        self.vbox.pack_start(oSubBox)

        self.show_all()

    def _radio_toggled(self, oRadioButton, oSubBox, oNewChild):
        """Display the correct sub-widgets when radio buttons change"""
        if oRadioButton.get_active():
            for oChild in oSubBox.get_children():
                oSubBox.remove(oChild)
            if oNewChild:
                oSubBox.pack_start(oNewChild)
            self.show_all()

    def get_url(self):
        """Get the url selected for download (or None)."""
        if self.oSourceWw.get_active():
            return dict(self.WW_RULEBOOK_URLS)
        elif self.oSourceUrl.get_active():
            return dict([(sName, self.dUrlSubEntries[sName].get_text())
                for sName in self.POSSIBLE_FILES])
        else:
            return None

    def get_file(self):
        """Get the file name selected (or None)."""
        if self.oSourceFile.get_active():
            return dict([(sName, self.dFileSubButtons[sName].get_filename())
                for sName in self.POSSIBLE_FILES])
        else:
            return None


class RulebookPlugin(CardListPlugin):
    """Plugin allowing downloading of rulebook HTML pages and making them
       available via the Plugins menu.
       """
    aModelsSupported = ["MainWindow"]

    POSSIBLE_FILES = RulebookConfigDialog.POSSIBLE_FILES
    WW_RULEBOOK_URLS = RulebookConfigDialog.WW_RULEBOOK_URLS

    LOCAL_NAMES = {
        "Rulebook": "rulebook.html",
        "Imbued Rules": "imbued.html",
        "Rulings": "rulings.html",
        "V:EKN Tournament Rules": "tournament_rules.html",
        "VTES FAQ" : "faq.html",
        "V:TES Complete Rules Reference": "rules_reference.html",
    }

    # pylint: disable-msg=W0142
    # ** magic OK here
    def __init__(self, *aArgs, **kwargs):
        super(RulebookPlugin, self).__init__(*aArgs, **kwargs)
        self._oRulebookDlg = None
        self._dMenuItems = None
        self._sPrefsPath = os.path.join(prefs_dir('Sutekh'), 'rulebook')

    def get_menu_item(self):
        """Overrides method from base class.

           Adds the menu item on the MainWindow if the files can be found.
           """
        if not self.check_versions() or not self.check_model_type():
            return None

        oSubMenu = gtk.Menu()
        oMenuItem = gtk.MenuItem("Rulebook")
        oMenuItem.set_submenu(oSubMenu)

        oConfigMenuItem = gtk.MenuItem("Configure Rulebook Plugin")
        oConfigMenuItem.connect("activate", self.config_activate)
        oSubMenu.add(oConfigMenuItem)

        self._dMenuItems = {}
        for sName in self.POSSIBLE_FILES:
            oItem = gtk.MenuItem(sName)
            oItem.connect("activate", self.rulebook_activate, sName)
            self._dMenuItems[sName] = oItem
            oSubMenu.add(oItem)

        self._update_menu()

        return [('Plugins', oMenuItem)]

    def _update_menu(self):
        """Check for existence of files and update which menu items are active
           accordingly.
           """
        for sName, sFile in self.LOCAL_NAMES.items():
            if not os.path.isfile(os.path.join(self._sPrefsPath, sFile)):
                self._dMenuItems[sName].set_sensitive(False)
            else:
                self._dMenuItems[sName].set_sensitive(True)

    def setup(self):
        """Prompt the user to download/setup the rulebook the first time"""
        if not os.path.exists(self._sPrefsPath):
            # Looks like the first time
            oDialog = RulebookConfigDialog(self.parent, True)
            self.handle_config_response(oDialog)
            # Don't get called next time
            ensure_dir_exists(self._sPrefsPath)

    # pylint: disable-msg=W0613
    # oMenuWidget needed by gtk function signature
    def config_activate(self, oMenuWidget):
        """Launch the configuration dialog."""
        oDialog = RulebookConfigDialog(self.parent, False)
        self.handle_config_response(oDialog)

    # pylint: disable-msg=W0613
    # oMenuWidget needed by gtk function signature
    def rulebook_activate(self, oMenuWidget, sName):
        """Show HTML file associated with the sName."""
        sPath = os.path.join(self._sPrefsPath, self.LOCAL_NAMES[sName])
        sPath = os.path.abspath(sPath)
        sDrive, sPath = os.path.splitdrive(sPath)
        # os.path.splitdrive assumes that drives have the form "X:"
        # so do the same in our replace.
        sPath = sDrive.replace(":", "|") + sPath
        sPath = sPath.replace(os.path.sep, "/")
        if not sPath.startswith("/"):
            sPath = "/" + sPath
        sUrl = "file:///" + sPath
        webbrowser.open(sUrl)

    def _link_resource(self, sLocalUrl):
        """Return a file-like object which sLocalUrl can be read from."""
        sResource = os.path.join(self._sPrefsPath, sLocalUrl)
        if os.path.exists(sResource):
            return file(sResource, 'rb')
        else:
            raise ValueError("Unknown resource %s" % sLocalUrl)

    def handle_config_response(self, oConfigDialog):
        """Handle the response from the config dialog"""
        iResponse = oConfigDialog.run()

        if iResponse == gtk.RESPONSE_OK:
            dUrls = oConfigDialog.get_url()
            dFiles = oConfigDialog.get_file()

            if dUrls is not None:
                if not self._download_files(dUrls):
                    do_complaint_error('Unable to successfully download '
                                        'some or all of the rulebook files')
            elif dFiles is not None:
                if not self._copy_files(dFiles):
                    do_complaint_error('Unable to successfully copy '
                                        'some or all of the rulebook files')
            else:
                # something weird happened
                pass

        self._update_menu()

        # get rid of the dialog
        oConfigDialog.destroy()

    def _download_files(self, dUrls):
        """Download files from a dictionary of sName -> sURL mappings."""
        bSuccess = True
        for sName, sUrl in dUrls.items():
            if not sUrl:
                continue
            bResult = self._download_rulebook(sName, sUrl)
            bSuccess = bSuccess and bResult
        return bSuccess

    def _download_rulebook(self, sName, sUrl):
        """Download a rulebook from a URL."""
        try:
            oFile = WwFile(sUrl, bUrl=True).open()
            sLength = oFile.info().getheader('Content-Length')
            if sLength:
                sData = ""
                iLength = int(sLength)
                oProgress = ProgressDialog()
                oProgress.set_description('Download progress')
                iTotal = 0
                bCont = True
                while bCont:
                    sInf = oFile.read(10000)
                    iTotal += 10000
                    oProgress.update_bar(float(iTotal) / iLength)
                    if sInf:
                        sData += sInf
                    else:
                        bCont = False
                        oProgress.destroy()
            else:
                # Just try and download
                sData = oFile.read()

            return self._save_rulebook(sName, sData)
        # pylint: disable-msg=W0703
        # We really do want to catch all exceptions here
        except Exception:
            return False

    def _copy_files(self, dFiles):
        """Copy files from a dictionary of sName -> sFile mappings."""
        bSuccess = True
        for sName, sFile in dFiles.items():
            if not sFile:
                continue
            bResult = self._copy_rulebook(sName, sFile)
            bSuccess = bSuccess and bResult
        return bSuccess

    def _copy_rulebook(self, sName, sFile):
        """Copy the rulebook from a local file."""
        try:
            oFile = WwFile(sFile).open()
            sData = oFile.read()
            return self._save_rulebook(sName, sData)
        # pylint: disable-msg=W0703
        # We really do want to catch all exceptions here
        except Exception:
            return False

    def _save_rulebook(self, sName, sData):
        """Save the data from a rulebook to the preferences folder."""
        sData = self._clean_rulebook(sName, sData)

        ensure_dir_exists(self._sPrefsPath)
        sFile = os.path.join(self._sPrefsPath, self.LOCAL_NAMES[sName])

        oFile = file(sFile, 'wb')
        oFile.write(sData)
        oFile.close()

        if os.path.isfile(sFile):
            return True
        return False

    @staticmethod
    def _clean_rulebook_tags(sData):
        """Clean up the tags in the rulebook HTML"""
        # img tags are always short tags in the WW rulebook file
        oShortTags = set(['hr', 'br', 'img'])

        oStartTagRe = re.compile(r"<\s*(?P<name>[A-Z0-9a-z_]+)"
            r"(?P<attrs>[^>]*)>",
            re.MULTILINE)
        oEndTagRe = re.compile(r"</\s*(?P<name>[A-Z0-9a-z_]+)\s*>",
            re.MULTILINE)
        oAttrRe = re.compile(r"(?P<attr>\w+)\s*=\s*" \
                             r"(?P<value>\"(?:[^\"]|\\\")*\"|'[^']*'|[^\s]*)",
            re.MULTILINE)

        def fix_attr(oMatch):
            """Fix attribute."""
            sName = oMatch.group('attr').lower()
            sValue = oMatch.group('value')
            if sValue.startswith("'") or sValue.endswith("'"):
                sValue = "'%s'" % (sValue.strip("'"),)
            else:
                sValue = "\"%s\"" % (sValue.strip("\""),)
            return "%s=%s" % (sName, sValue)

        def fix_start_tag(oMatch):
            """Standardize start tags."""
            sName = oMatch.group('name').lower()
            sAttrs = oMatch.group('attrs')
            sAttrs = oAttrRe.sub(fix_attr, sAttrs)
            if sName in oShortTags and not sAttrs.endswith("/"):
                return "<%s%s/>" % (sName, sAttrs)
            else:
                return "<%s%s>" % (sName, sAttrs)

        def fix_end_tag(oMatch):
            """Standardize end tags."""
            sName = oMatch.group('name').lower()
            return "</%s>" % (sName,)

        sData = oStartTagRe.sub(fix_start_tag, sData)
        sData = oEndTagRe.sub(fix_end_tag, sData)

        return sData

    # pylint: disable-msg=R0914
    # We use several local variables for clarity
    def _clean_rulebook(self, sName, sData):
        """Clean up rulebook HTML"""
        sData = self._clean_rulebook_tags(sData)

        # in rulings, remove everything between header and first
        # end of first style tag
        if sName in ("Rulings", "Imbued Rules", "V:EKN Tournament Rules",
            "V:TES Complete Rules Reference"):
            oHdrToStyle = re.compile(r"</head>.*?</style>",
                re.MULTILINE | re.DOTALL)
            sData = oHdrToStyle.sub("</head>", sData, 1)

        # remove <script> tags since they're not
        # vital to the rulebook and avoid attempts by the
        # browser to access the network
        oTagRemove = re.compile(r"<script[^<>]*>[^<]*"
            r"(</script[^<>]*>)?",
            re.MULTILINE)
        sData = oTagRemove.sub("", sData)
        # remove <img> tags. These are not closed properly
        oTagRemove = re.compile(r"<img[^<>]*>")
        sData = oTagRemove.sub("", sData)
        # remove the onclick tags from the imbued rules, since they're
        # just broken
        oClickRemove = re.compile(r'\(?<a href="(images|#)[^>]*onclick=.*>'
            r'.*</a>\)?',
            re.MULTILINE)
        sData = oClickRemove.sub("", sData)

        # unclosed <a name=...>
        oUnclosedName = re.compile(r"(<a name=[^>]*>[^<>]*"
            r"<h.>[^<>]*</h.>[^<>]*)<p>",
            re.MULTILINE)

        sData = oUnclosedName.sub(r"\g<1></a><p>", sData)

        # base url for href's that begin with . or /
        sFullUrl = self.WW_RULEBOOK_URLS[sName]
        sBaseUrl = sFullUrl.rsplit("/", 1)[0]
        oHref = re.compile(r"(?P<start>href=[\"'])(?P<href>[^\"']*)"
                            r"(?P<end>[\"'])")

        def apply_base_url(oMatch):
            """Apply a base url to hrefs that start with . or /"""
            sHref = oMatch.group("href")
            if sHref.startswith(".") or sHref.startswith("/"):
                sHref = urllib.basejoin(sBaseUrl, sHref)
            return oMatch.group("start") + sHref + oMatch.group("end")

        sData = oHref.sub(apply_base_url, sData)

        # translate rulebook URLs to local links
        for sName, sUrl in self.WW_RULEBOOK_URLS.items():
            sLocal = self.LOCAL_NAMES[sName]
            sData = sData.replace(sUrl, sLocal)

        return sData

# pylint: disable-msg=C0103
# shut up complaint about the name
plugin = RulebookPlugin



# test_JOLDeckParser.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Test reading a card set from an JOL deck file"""

import unittest
from sutekh.tests.TestCore import SutekhTest, DummyHolder
from sutekh.io.JOLDeckParser import JOLDeckParser

class TestJOLDeckParser(SutekhTest):
    """class for the JOL deck file parser tests"""

    sTestText1 = """
        3xTest Vamp 1
        Alan Sovereign (advanced)
        The Path of Blood
        Path of Blood, The
        Test Vamp 2
        Test Card 1
        Test Card 1
        4 x   Test Card 2
        Test Card 3
        Test Card 3
        Test Card 3
        Test Card 3
        Test Card 3
        Test Card 3
        Test Card 3
        Test Card 3
        Test Card 3
        Test Card 3
        Test Card 3
        Test Card 3
        Test Card 4
        12x Test Card 5
        2 xTest Card 6
        """

    def test_basic(self):
        """Run the JOL input test."""
        oHolder = DummyHolder()
        oParser = JOLDeckParser(oHolder)

        for sLine in self.sTestText1.split("\n"):
            oParser.feed(sLine + "\n")

        aCards = oHolder.get_cards()

        self.assertEqual(len(aCards), 10)
        self.failUnless(("Test Vamp 1", 3) in aCards)
        self.failUnless(("Test Vamp 2", 1) in aCards)
        self.failUnless(("Test Card 1", 2 ) in aCards)
        self.failUnless(("Test Card 2", 4) in aCards)
        self.failUnless(("Test Card 3", 12) in aCards)
        self.failUnless(("Test Card 4", 1) in aCards)
        self.failUnless(("Test Card 5", 12) in aCards)
        self.failUnless(("Test Card 6", 2) in aCards)
        self.failUnless(("The Path of Blood", 2) in aCards)
        self.failUnless(("Alan Sovereign (Advanced)", 1) in aCards)

if __name__ == "__main__":
    unittest.main()



# test_ARDBXMLDeckParser.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Test reading a card set from an ARDB XML deck file"""

import unittest
from sutekh.tests.TestCore import DummyHolder
from sutekh.io.ARDBXMLDeckParser import ARDBXMLDeckParser

class ArdbXMLDeckParserTests(unittest.TestCase):
    """class for the ARDB XML deck file parser tests"""

    # ARDB produces tag pairs for empty elements, we produce minimal
    # tags (<set></set> vs <set />, so we have both in the test data
    sTestText1 = """
    <deck generator="Sutekh [ Test ]" fromatVersion="val">
        <name>Test Deck</name>
        <author>Anon Y Mous</author>
        <description>Simple test deck.

        http://www.example.url/in/description</description>
        <date>2008-09-01</date>
        <crypt size="3" min="1" max="2" avg="1.1">
            <vampire databaseID="1" count="2">
               <name>Test Vamp 1</name>
               <adv></adv>
               <set>CE</set>
               <capacity></capacity>
               <text></text>
               <group />
               <disciplines></disciplines>
            </vampire>
            <vampire databaseID="2" count="1">
               <name>Test Vamp 2</name>
               <adv></adv>
               <set>SW</set>
               <capacity></capacity>
               <text></text>
               <group></group>
               <disciplines></disciplines>
            </vampire>
            <vampire databaseID="2" count="1">
               <name>Test Vamp 2</name>
               <adv>(Advanced)</adv>
               <set>Promo20051001</set>
               <capacity></capacity>
               <text></text>
               <group></group>
               <disciplines></disciplines>
            </vampire>
        </crypt>
        <library size="17">
           <card databaseID="3" count="4">
              <name>Test Card 1</name>
              <set>Sabbat</set>
              <type></type>
              <cost></cost>
              <requirement></requirement>
              <text />
           </card>
           <card databaseID="3" count="2">
              <name>Test Card 2</name>
              <set>BH</set>
              <type></type>
              <cost></cost>
              <requirement />
              <text></text>
           </card>
           <card databaseID="3" count="12">
              <name>Test Card 3</name>
              <set>BH</set>
              <type></type>
              <cost></cost>
              <requirement></requirement>
              <text></text>
           </card>
           <card databaseID="3" count="1">
              <name>Test Card 4</name>
              <set></set>
              <type></type>
              <cost></cost>
              <requirement></requirement>
              <text></text>
           </card>
           <card databaseID="50" count="1">
              <name>Test Card 5, The</name>
              <set></set>
              <type></type>
              <cost></cost>
              <requirement></requirement>
              <text></text>
           </card>
        </library>
    </deck>
    """

    def test_basic(self):
        """Run the input test."""
        oHolder = DummyHolder()
        oParser = ARDBXMLDeckParser(oHolder)

        for sLine in self.sTestText1.split("\n"):
            oParser.feed(sLine + "\n")

        self.assertEqual(oHolder.name, "Test Deck")
        self.assertEqual(oHolder.author, "Anon Y Mous")
        self.failUnless(oHolder.comment.startswith("Simple test deck."))
        self.failUnless(oHolder.comment.endswith("in/description"))

        aCards = oHolder.get_cards_exps()

        self.assertEqual(len(aCards), 8)
        self.failUnless((("Test Vamp 1", "CE"), 2) in aCards)
        self.failUnless((("Test Vamp 2", "SW"), 1) in aCards)
        self.failUnless((("Test Vamp 2 (Advanced)", "Promo-20051001"), 1)
                in aCards)
        self.failUnless((("Test Card 1", "Sabbat"), 4) in aCards)
        self.failUnless((("Test Card 2", "BH"), 2) in aCards)
        self.failUnless((("Test Card 3", "BH"), 12) in aCards)
        self.failUnless((("Test Card 4", None), 1) in aCards)
        self.failUnless((("The Test Card 5", None), 1) in aCards)


if __name__ == "__main__":
    unittest.main()



# test_ARDBXMLInvParser.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Test reading a card set from an ARDB XML inventory file"""

import unittest
from sutekh.tests.TestCore import DummyHolder
from sutekh.io.ARDBXMLInvParser import ARDBXMLInvParser

class ArdbXMLInvParserTests(unittest.TestCase):
    """class for the ARDB XML inventory file parser tests"""

    # ARDB produces tag pairs for empty elements, we produce minimal
    # tags (<set></set> vs <set />, so we have both in the test data
    sTestText1 = """
    <inventory generator="Sutekh [ Test ]" fromatVersion="val">
        <date>2008-09-01</date>
        <crypt size="3">
            <vampire databaseID="1" have="2" spare="0" need="2">
               <name>Test Vamp 1</name>
               <adv></adv>
               <set>CE</set>
               <rarity>U2</rarity>
            </vampire>
            <vampire databaseID="2" have="1" spare="0" need="0">
               <name>Test Vamp 2</name>
               <adv></adv>
               <set>SW</set>
               <rarity>U</rarity>
            </vampire>
            <vampire databaseID="12" have="1" spare="0" needs="0">
               <name>Alan Sovereign</name>
               <adv>(Advanced)</adv>
               <set>Promo20051001</set>
               <rarity>P</rarity>
            </vampire>
        </crypt>
        <library size="17">
           <card databaseID="3" have="4" spare="0" need="0">
              <name>Test Card 1</name>
              <set>Sabbat</set>
              <rarity>C</rarity>
           </card>
           <card databaseID="3" have="2" spare="0" need="0">
              <name>Test Card 2</name>
              <set>BH</set>
              <rarity>C</rarity>
           </card>
           <card databaseID="3" have="12" spare="0" need="0">
              <name>Test Card 3</name>
              <set>BH</set>
              <rarity>C</rarity>
           </card>
           <card databaseID="3" have="1" spare="0" need="0">
              <name>Test Card 4</name>
              <set></set>
              <rarity>C</rarity>
           </card>
           <card databaseID="30" have="1" spare="0" need="0">
              <name>Path of Blood, The</name>
              <set></set>
              <rarity>C</rarity>
           </card>

        </library>
    </inventory>
    """

    def test_basic(self):
        """Run the input test."""
        oHolder = DummyHolder()
        oParser = ARDBXMLInvParser(oHolder)

        for sLine in self.sTestText1.split("\n"):
            oParser.feed(sLine + "\n")

        self.assertEqual(oHolder.name, "")

        aCards = oHolder.get_cards_exps()

        self.assertEqual(len(aCards), 8)
        self.failUnless((("Test Vamp 1", "CE"), 2) in aCards)
        self.failUnless((("Test Vamp 2", "SW"), 1) in aCards)
        self.failUnless((("Test Card 1", "Sabbat"), 4) in aCards)
        self.failUnless((("Test Card 2", "BH"), 2) in aCards)
        self.failUnless((("Test Card 3", "BH"), 12) in aCards)
        self.failUnless((("Test Card 4", None), 1) in aCards)
        self.failUnless((("The Path of Blood", None), 1) in aCards)
        self.failUnless((("Alan Sovereign (Advanced)", 'Promo-20051001'), 1)
                in aCards)

if __name__ == "__main__":
    unittest.main()



# SutekhCli.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2005, 2006 Simon Cross <hodgestar@gmail.com>
# Copyright 2006 Neil Muller <drnlmuller+sutekh@gmail.com> (minor tweaks)
# GPL - see COPYING for details
"""
SutekhCli.py: command-line interface to much of Sutekh's database
management functionality
"""

import sys, optparse, os
from logging import StreamHandler
from sqlobject import sqlhub, connectionForURI
from sutekh.core.SutekhObjects import Ruling, aObjectList, aPhysicalList
from sutekh.SutekhUtility import refresh_tables, read_white_wolf_list, \
        read_rulings, gen_temp_dir, prefs_dir, ensure_dir_exists, sqlite_uri
from sutekh.core.DatabaseUpgrade import attempt_database_upgrade
from sutekh.io.XmlFileHandling import PhysicalCardXmlFile, \
        PhysicalCardSetXmlFile, AbstractCardSetXmlFile, \
        write_all_pcs
from sutekh.io.ZipFileWrapper import ZipFileWrapper
from sutekh.io.WwFile import WwFile
from sutekh.SutekhInfo import SutekhInfo

def parse_options(aArgs):
    """Handle the command line options"""
    oOptParser = optparse.OptionParser(usage="usage: %prog [options]",
            version="%%prog %s" % SutekhInfo.VERSION_STR)
    oOptParser.add_option("-d", "--db",
                  type="string", dest="db", default=None,
                  help="Database URI. [sqlite://$PREFSDIR$/sutekh.db]")
    oOptParser.add_option("-r", "--ww-file",
                  type="string", dest="ww_file", default=None,
                  help="HTML file (probably from WW website) to read " \
                          "cards from.")
    oOptParser.add_option("--ruling-file",
                  type="string", dest="ruling_file", default=None,
                  help="HTML file (probably from WW website) to read " \
                          "rulings from.")
    oOptParser.add_option("-c", "--refresh-tables",
                  action="store_true", dest="refresh_tables", default=False,
                  help="Drop (if possible) and recreate database tables.")
    oOptParser.add_option("--refresh-ruling-tables",
                  action="store_true", dest="refresh_ruling_tables",
                  default=False,
                  help="Drop (if possible) and recreate rulings tables only.")
    oOptParser.add_option("--refresh-physical-card-tables",
                  action="store_true", dest="refresh_physical_card_tables",
                  default=False,
                  help="Drop (if possible) and recreate physical card " \
                          "tables only.")
    oOptParser.add_option("--sql-debug",
                  action="store_true", dest="sql_debug", default=False,
                  help="Print out SQL statements.")
    oOptParser.add_option("-l", "--read-physical-cards-from",
                  type="string", dest="read_physical_cards_from", default=None,
                  help="Read physical card list from the given XML file.")
    oOptParser.add_option("--save-pcs",
                  type="string", dest="save_pcs", default=None,
                  help="Save the given Physical Card Set to an XML file " \
                          "(by default named <pcsname>.xml).")
    oOptParser.add_option("--pcs-filename",
                  type="string", dest="pcs_filename", default=None,
                  help="Give an alternative filename to save the Physical " \
                          "Card Set as")
    oOptParser.add_option("--save-all-pcs",
                  action="store_true", dest="save_all_pcss", default=False,
                  help="Save all Physical Card Sets in the database to files" \
                          " - Cannot be used with --save-pcs.")
    oOptParser.add_option("--read-pcs",
                  type="string", dest="read_pcs", default=None,
                  help="Load a Physical Card Set from the given XML file.")
    oOptParser.add_option("--read-acs",
                  type="string", dest="read_acs", default=None,
                  help="Load an Abstract Card Set from the given XML file.")
    oOptParser.add_option("--reload", action="store_true", dest="reload",
                  default=False,
                  help="Dump all card sets and reload them - intended to be" \
                          " used with -c and refreshing the abstract card" \
                          " list")
    oOptParser.add_option("--upgrade-db",
                  action="store_true", dest="upgrade_db", default=False,
                  help="Attempt to upgrade a database to the latest version." \
                          " Cannot be used with --refresh-tables")
    oOptParser.add_option("--dump-zip",
                  type="string", dest="dump_zip_name", default=None,
                  help="Dump the all the card sets to the given zip file")
    oOptParser.add_option("--restore-zip",
            type="string", dest="restore_zip_name", default=None,
            help="Restore everything from the given zipfile")

    return oOptParser, oOptParser.parse_args(aArgs)

def main(aArgs):
    """
    Main function: Loop through the options and process the database
    accordingly.
    """
    # Turn off some pylint refactoring warnings
    # pylint: disable-msg=R0915, R0912, R0911
    oOptParser, (oOpts, aArgs) = parse_options(aArgs)
    sPrefsDir = prefs_dir("Sutekh")

    oLogHandler = StreamHandler(sys.stdout)

    if len(aArgs) != 1:
        oOptParser.print_help()
        return 1

    if oOpts.db is None:
        ensure_dir_exists(sPrefsDir)
        oOpts.db = sqlite_uri(os.path.join(sPrefsDir, "sutekh.db"))

    oConn = connectionForURI(oOpts.db)
    sqlhub.processConnection = oConn

    if oOpts.sql_debug:
        oConn.debug = True

    if oOpts.reload:
        if not oOpts.refresh_tables:
            print "reload should be called with --refresh-tables"
            return 1
        else:
            sTempdir = gen_temp_dir()
            aPhysicalCardSetList = write_all_pcs(sTempdir)
            oPCFile = PhysicalCardXmlFile(dir=sTempdir)
            oPCFile.write()
            # We dump the databases here
            # We will reload them later

    if oOpts.refresh_ruling_tables:
        if not refresh_tables([Ruling], sqlhub.processConnection):
            print "refresh failed"
            return 1

    if oOpts.refresh_tables:
        if not refresh_tables(aObjectList, sqlhub.processConnection):
            print "refresh failed"
            return 1

    if oOpts.refresh_physical_card_tables:
        if not refresh_tables(aPhysicalList, sqlhub.processConnection):
            print "refresh failed"
            return 1

    if not oOpts.ww_file is None:
        read_white_wolf_list([WwFile(oOpts.ww_file)], oLogHandler)

    if not oOpts.ruling_file is None:
        read_rulings(WwFile(oOpts.ruling_file), oLogHandler)

    if not oOpts.read_physical_cards_from is None:
        oFile = PhysicalCardXmlFile(oOpts.read_physical_cards_from)
        oFile.read()

    if oOpts.save_all_pcss and not oOpts.save_pcs is None:
        print "Can't use --save-pcs and --save-all-pcs Simulatenously"
        return 1

    if oOpts.save_all_pcss:
        write_all_pcs()

    if oOpts.dump_zip_name is not None:
        oZipFile = ZipFileWrapper(oOpts.dump_zip_name)
        oZipFile.do_dump_all_to_zip(oLogHandler)

    if oOpts.restore_zip_name is not None:
        oZipFile = ZipFileWrapper(oOpts.restore_zip_name)
        oZipFile.do_restore_from_zip(oLogHandler=oLogHandler)

    if not oOpts.save_pcs is None:
        oFile = PhysicalCardSetXmlFile(oOpts.pcs_filename)
        oFile.write(oOpts.save_pcs)

    if not oOpts.read_pcs is None:
        oFile = PhysicalCardSetXmlFile(oOpts.read_pcs)
        oFile.read()

    if not oOpts.read_acs is None:
        oFile = AbstractCardSetXmlFile(oOpts.read_acs)
        oFile.read()

    if oOpts.reload:
        oPCFile.read()
        oPCFile.delete()
        for oPCSet in aPhysicalCardSetList:
            oPCSet.read()
            oPCSet.delete()
        os.rmdir(sTempdir)

    if oOpts.upgrade_db and oOpts.refresh_tables:
        print "Can't use --upgrade-db and --refresh-tables simulatenously"
        return 1

    if oOpts.upgrade_db:
        attempt_database_upgrade(oLogHandler)

    return 0

if __name__ == "__main__":
    sys.exit(main(sys.argv))



# test_Filters.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2007 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details

"""Sutekh Filters tests"""

from sutekh.tests.TestCore import SutekhTest
from sutekh.tests.io import test_WhiteWolfParser
from sutekh.core.SutekhObjects import AbstractCard, IAbstractCard, \
        PhysicalCard, IPhysicalCard, Expansion, IExpansion, PhysicalCardSet, \
        MapPhysicalCardToPhysicalCardSet
from sutekh.core import Filters
from sqlobject import SQLObjectNotFound
from sutekh.core.CardLookup import best_guess_filter
import unittest

class FilterTests(SutekhTest):
    """Test class for testing Sutekh Filters"""
    aExpectedCards = test_WhiteWolfParser.WhiteWolfParserTests.aExpectedCards

    def _physical_test(self, tTest):
        """Convert the tuple describing the test as a filter and a list of
           card names and optional expansions into the correct
           filter on the physical card list and a list of expected
           PhysicalCard objects."""
        if len(tTest) == 2:
            oFilter, aExpectedNames = tTest
            aAllowedExpansions = set(Expansion.select())
            aAllowedExpansions.add(None)
        else:
            assert len(tTest) == 3
            oFilter, aExpectedNames = tTest[:2]
            aAllowedExpansions = set([IExpansion(sExp) for sExp in tTest[2]
                                        if sExp is not None])
            if None in tTest[2]:
                aAllowedExpansions.add(None)

        aPhysicalCards = []
        for sName in aExpectedNames:
            # pylint: disable-msg=E1101
            # sqlobject confuses pylint
            oAbs = IAbstractCard(sName)
            aExps = set([oRarity.expansion for oRarity in oAbs.rarity])

            if None in aAllowedExpansions:
                aPhysicalCards.append(IPhysicalCard((oAbs, None)))

            for oExp in aExps:
                if not oExp in aAllowedExpansions:
                    continue
                try:
                    aPhysicalCards.append(IPhysicalCard((oAbs, oExp)))
                except SQLObjectNotFound:
                    self.fail(
                        "Missing physical card %s from expansion %s"
                        % (oAbs.name, oExp.name)
                    )

        oFullFilter = Filters.FilterAndBox([Filters.PhysicalCardFilter(),
                                            oFilter])

        return oFullFilter, sorted(aPhysicalCards)

    def _convert_to_phys_cards(self, aList):
        """Converts a list of Name, Expansion Name tuples into physical
           cards"""
        aPhysCards = []
        for sName, sExp in aList:
            try:
                if sExp:
                    oExp = IExpansion(sExp)
                else:
                    oExp = None
                oAbs = IAbstractCard(sName)
                oCard = IPhysicalCard((oAbs, oExp))
                aPhysCards.append(oCard)
            except SQLObjectNotFound:
                self.fail("Invalid physical card (%s from expansion %s)"
                        % (sName, sExp))
        return sorted(aPhysCards)

    # pylint: disable-msg=R0914
    # We don't really care about the number of local variables here
    def test_basic(self):
        """Simple tests of the filter"""
        # setup filters
        aTests = [
            # Single / Multi Filters
            (Filters.ClanFilter('Follower of Set'), [u"Aabbt Kindred",
                u"Abdelsobek", u"Amisa", u"Kemintiri (Advanced)"]),
            (Filters.MultiClanFilter(['Ravnos', 'Samedi']), [u"Abebe",
                u"L\xe1z\xe1r Dobrescu"]),
            (Filters.DisciplineFilter('obf'), [u"Aaron Bathurst",
                u"Abd al-Rashid", u"Abdelsobek", u"Abebe", u"Aeron", u"Amisa",
                u"Angelica, The Canonicus", u"Cedric", u"Kabede Maru",
                u"Kemintiri (Advanced)", u"Sha-Ennu"]),
            (Filters.DisciplineFilter('fli'), [u"Cedric"]),
            (Filters.MultiDisciplineFilter(['nec', 'qui']), [u"Abd al-Rashid",
                u"Abdelsobek", u"Abebe", u"Akram", u"Ambrogino Giovanni",
                u"Kabede Maru"]),
            (Filters.ExpansionFilter('NoR'), [u"Abjure",
                u'Anna "Dictatrix11" Suljic', u'Earl "Shaka74" Deams',
                u'Inez "Nurse216" Villagrande']),
            (Filters.MultiExpansionFilter(['NoR', 'LoB']), [u".44 Magnum",
                u"Abebe", u"Abjure", u"Abombwe",
                u'Anna "Dictatrix11" Suljic', u"Cedric", u"Cesewayo",
                u'Earl "Shaka74" Deams', u'Inez "Nurse216" Villagrande',
                u"Predator's Communion"]),
            (Filters.ExpansionRarityFilter(('Sabbat', 'Rare')),
                [u"Ablative Skin"]),
            (Filters.ExpansionRarityFilter(('Blood Shadowed Court',
                'BSC')), [u"Alan Sovereign", u"Alfred Benezri",
                    u"Anastasz di Zagreb", u"Gracis Nostinus",
                    u"Yvette, The Hopeless"]),
            (Filters.MultiExpansionRarityFilter([('Third', 'Uncommon'),
                ('Jyhad', 'Rare')]), [u"Aaron's Feeding Razor", u"Abbot"]),
            (Filters.DisciplineLevelFilter(('cel', 'superior')),
                [u"Abd al-Rashid", u"Akram", u"Alexandra", u"Anson",
                    u"Bronwen", u"Cesewayo", u"Kabede Maru"]),
            (Filters.MultiDisciplineLevelFilter([('obt', 'inferior'),
                ('pot', 'inferior'), ('obf', 'superior')]),
                [u"Aaron Bathurst", u"Aaron Duggan, Cameron's Toady",
                    u"Aeron", u"Akram", u"Amisa", u"Bronwen", u"Cedric",
                    u"Kabede Maru", "Kemintiri (Advanced)"]),
            (Filters.MultiDisciplineLevelFilter(['obt with inferior',
                'pot with inferior', 'obf with superior']),
                [u"Aaron Bathurst", u"Aaron Duggan, Cameron's Toady",
                    u"Aeron", u"Akram", u"Amisa", u"Bronwen", u"Cedric",
                    u"Kabede Maru", u"Kemintiri (Advanced)"]),
            (Filters.CardTypeFilter('Equipment'), [u".44 Magnum", u"AK-47",
                u"Aaron's Feeding Razor"]),
            (Filters.CardTypeFilter('Reflex'), [u"Predator's Communion"]),
            (Filters.MultiCardTypeFilter(['Power', 'Action']), [u"Abbot",
                u"Abjure", u"Ablative Skin"]),
            (Filters.SectFilter('Sabbat'), [u"Aaron Bathurst",
                u"Aaron Duggan, Cameron's Toady", u"Aeron", u"Alfred Benezri",
                u"Angelica, The Canonicus", u"Bronwen", u"Sha-Ennu"]),
            (Filters.MultiSectFilter(['Sabbat', 'Independent']),
                [u"Aabbt Kindred",
                u"Aaron Bathurst", u"Aaron Duggan, Cameron's Toady",
                u"Abd al-Rashid", u"Abdelsobek", u"Abebe",
                u"Aeron", u"Alfred Benezri", u"Ambrogino Giovanni", u"Amisa",
                u"Angelica, The Canonicus", u"Bronwen",
                u"Kemintiri (Advanced)", u"L\xe1z\xe1r Dobrescu",
                u"Sha-Ennu"]),
            (Filters.TitleFilter('Bishop'), [u"Alfred Benezri"]),
            (Filters.TitleFilter('Independent with 1 vote'),
                [u"Ambrogino Giovanni"]),
            (Filters.MultiTitleFilter(['Bishop', 'Prince']),
                    [u"Alfred Benezri", u"Anson"]),
            (Filters.MultiTitleFilter(['Magaji', 'Regent']),
                    [u"Cesewayo", u"Kabede Maru", u"Sha-Ennu"]),
            (Filters.MultiTitleFilter(['Primogen', 'Priscus', 'Cardinal']),
                    [u"Akram", u"Angelica, The Canonicus", u"Bronwen"]),
            (Filters.CreedFilter('Martyr'),
                    [u'Anna "Dictatrix11" Suljic']),
            (Filters.MultiCreedFilter(['Martyr', 'Innocent']),
                    [u'Anna "Dictatrix11" Suljic',
                        u'Inez "Nurse216" Villagrande']),
            (Filters.VirtueFilter('Redemption'), [u"Abjure",
                u'Anna "Dictatrix11" Suljic']),
            #(Filters.MultiVirtueFilter(['Redemption', 'Judgement']),
            #    [u"Abjure"]),
            (Filters.GroupFilter(4), [u"Aaron Bathurst", u"Abebe",
                u'Anna "Dictatrix11" Suljic', u"Cedric", u"Cesewayo",
                u'Earl "Shaka74" Deams', u'Inez "Nurse216" Villagrande',
                u"Sha-Ennu"]),
            (Filters.MultiGroupFilter([4, 5]), [u"Aaron Bathurst",
                u"Abdelsobek", u"Abebe", u'Anna "Dictatrix11" Suljic',
                u"Cedric", u"Cesewayo", u'Earl "Shaka74" Deams',
                u'Inez "Nurse216" Villagrande', u"Kabede Maru", u"Sha-Ennu"]),
            (Filters.CapacityFilter(2), [u"Aaron Duggan, Cameron's Toady"]),
            (Filters.MultiCapacityFilter([2, 1]),
                [u"Aaron Duggan, Cameron's Toady", u"Abombwe"]),
            (Filters.CostFilter(5), [u"AK-47"]),
            (Filters.MultiCostFilter([2, 5]), [u".44 Magnum", u"AK-47"]),
            (Filters.MultiCostFilter([0]), [u'Aabbt Kindred',
                u'Aaron Bathurst', u"Aaron Duggan, Cameron's Toady",
                u'Abandoning the Flesh', u'Abbot', u'Abd al-Rashid',
                u'Abdelsobek', u'Abebe', u'Abjure', u'Ablative Skin',
                u'Abombwe', u'Aeron', u'Akram', u'Alan Sovereign',
                u'Alan Sovereign (Advanced)', u'Alexandra', u'Alfred Benezri',
                u'Ambrogino Giovanni', u'Amisa', u'Anastasz di Zagreb',
                u'Angelica, The Canonicus', u'Anna "Dictatrix11" Suljic',
                u'Anson', u'Bronwen', u'Cedric', u'Cesewayo',
                u'Earl "Shaka74" Deams', u'Gracis Nostinus',
                u'Inez "Nurse216" Villagrande', u'Kabede Maru',
                u'Kemintiri (Advanced)', u'L\xe1z\xe1r Dobrescu',
                u"Predator's Communion", u'Sha-Ennu',
                u'Yvette, The Hopeless']),
            (Filters.MultiCostFilter([0, 5]), [u"AK-47", u'Aabbt Kindred',
                u'Aaron Bathurst', u"Aaron Duggan, Cameron's Toady",
                u'Abandoning the Flesh', u'Abbot', u'Abd al-Rashid',
                u'Abdelsobek', u'Abebe', u'Abjure', u'Ablative Skin',
                u'Abombwe', u'Aeron', u'Akram', u'Alan Sovereign',
                u'Alan Sovereign (Advanced)', u'Alexandra', u'Alfred Benezri',
                u'Ambrogino Giovanni', u'Amisa', u'Anastasz di Zagreb',
                u'Angelica, The Canonicus', u'Anna "Dictatrix11" Suljic',
                u'Anson', u'Bronwen', u'Cedric', u'Cesewayo',
                u'Earl "Shaka74" Deams', u'Gracis Nostinus',
                u'Inez "Nurse216" Villagrande', u'Kabede Maru',
                u'Kemintiri (Advanced)', u'L\xe1z\xe1r Dobrescu',
                u"Predator's Communion", u'Sha-Ennu',
                u'Yvette, The Hopeless']),
            (Filters.LifeFilter(6), [u'Anna "Dictatrix11" Suljic',
                u'Earl "Shaka74" Deams']),
            (Filters.MultiLifeFilter([3, 6]), [u'Anna "Dictatrix11" Suljic',
                u'Earl "Shaka74" Deams', u'Inez "Nurse216" Villagrande']),
            (Filters.CostTypeFilter('Pool'), [u".44 Magnum", u"AK-47",
                u"Aaron's Feeding Razor", u"The Path of Blood"]),
            (Filters.CostTypeFilter('Blood'), [u"Aire of Elation"]),
            (Filters.MultiCostTypeFilter(['Pool', 'Blood']), [u".44 Magnum",
                u"AK-47", u"Aaron's Feeding Razor", u"Aire of Elation",
                U"The Path of Blood"]),
            (Filters.LifeFilter(4), []),
            (Filters.MultiLifeFilter([4, 5]), []),

            # Other Filters
            (Filters.CardTextFilter('strike'), [u".44 Magnum", u"AK-47",
                u"Aeron", u"Anastasz di Zagreb", u"Bronwen"]),
            (Filters.CardNameFilter(u'L\xe1z\xe1r'),
                [u"L\xe1z\xe1r Dobrescu"]),
            (Filters.NullFilter(), self.aExpectedCards),
            (Filters.SpecificCardFilter(IAbstractCard("Abebe")),
                [u"Abebe"]),

            # Compound Filters
            (Filters.FilterAndBox([Filters.CardTypeFilter('Equipment'),
                Filters.CostFilter(5)]),
                [u"AK-47"]),
            (Filters.FilterOrBox([Filters.CardTypeFilter('Equipment'),
                Filters.CardTypeFilter('Power')]), [u".44 Magnum", u"AK-47",
                    u"Aaron's Feeding Razor", u"Abjure"]),
            (Filters.FilterNot(Filters.MultiCardTypeFilter(['Equipment',
                'Vampire'])), [u"Abandoning the Flesh", u"Abbot", u"Abjure",
                    u"Ablative Skin", u"Abombwe", u"Aire of Elation",
                    u'Anna "Dictatrix11" Suljic', u'Earl "Shaka74" Deams',
                    u'Inez "Nurse216" Villagrande', u"Predator's Communion",
                    u"The Path of Blood"]),
        ]

        aPhysicalTests = [self._physical_test(tTest) for tTest in aTests]

        # Abstract Card Filtering Tests
        for oFilter, aExpectedNames in aTests:
            aCards = oFilter.select(AbstractCard).distinct()
            aNames = sorted([oC.name for oC in aCards])
            self.assertEqual(aNames, aExpectedNames, "Filter Object %s"
                    " failed. %s != %s." % (oFilter, aNames, aExpectedNames))

        # Filter values Tests
        self.assertEqual(Filters.MultiClanFilter.get_values(), [u"Assamite",
            u"Brujah", u"Brujah antitribu", u"Follower of Set", u"Gargoyle",
            u"Giovanni", u"Lasombra", u"Nosferatu antitribu", u"Osebo",
            u"Pander", u"Ravnos", u"Samedi", u"Toreador", u"Tremere",
            u"Tzimisce", u"Ventrue"])
        self.assertEqual(Filters.MultiDisciplineFilter.get_values(),
                [u"Abombwe", u"Animalism", u"Auspex", u"Celerity",
                    u"Chimerstry", u"Dementation", u"Dominate", u"Flight",
                    u"Fortitude", u"Necromancy", u"Obfuscate",
                    u"Obtenebration", u"Potence", u"Presence",
                    u"Protean", u"Quietus", u"Serpentis", u"Thaumaturgy",
                    u"Thanatosis", u"Vicissitude", u"Visceratika"])
        self.assertEqual(Filters.MultiCardTypeFilter.get_values(), [u"Action",
            u"Action Modifier", u"Combat", u"Equipment", u"Imbued", u"Master",
            u"Power", u"Reaction", u"Reflex", u"Vampire"])
        self.assertEqual(Filters.MultiTitleFilter.get_values(),
                [u"Archbishop", u"Bishop", u"Cardinal",
                    u"Independent with 1 vote", u"Independent with 2 votes",
                    u"Inner Circle", u"Justicar", u"Magaji", u"Primogen",
                    u"Prince", u"Priscus", u"Regent"])
        self.assertEqual(Filters.MultiCreedFilter.get_values(),
                [u"Innocent", u"Martyr", u"Visionary"])
        self.assertEqual(Filters.MultiVirtueFilter.get_values(),
                [u"Innocence", u"Judgment", u"Martyrdom", u"Redemption",
                    u"Vision"])

        # Test the physical card filtering
        for oFilter, aExpectedCards in aPhysicalTests:
            aCards = sorted(oFilter.select(PhysicalCard).distinct())
            self.assertEqual(aCards, aExpectedCards, "Filter Object %s"
                    " failed. %s != %s." % (oFilter, aCards, aExpectedCards))

        # test filtering on expansion
        aExpansionTests = [
                (Filters.PhysicalExpansionFilter('Jyhad'),
                    ['.44 Magnum', "Aaron's Feeding Razor", u"Anson"],
                    ['Jyhad']),
                (Filters.PhysicalExpansionFilter('LoB'),
                    ['Abombwe','.44 Magnum', 'Abebe', u"Cedric", u"Cesewayo",
                        u"Predator's Communion"], ['LoB']),
                (Filters.PhysicalExpansionFilter(None),
                    self.aExpectedCards,
                    [None]),
                (Filters.MultiPhysicalExpansionFilter(['LoB', 'LotN']),
                    ['Abombwe','.44 Magnum', 'Abebe', 'AK-47', 'Abdelsobek',
                        u"Cedric", u"Cesewayo", u"Kabede Maru",
                        u"Predator's Communion", u"The Path of Blood"],
                    ['LoB', 'LotN']),
                (Filters.MultiPhysicalExpansionFilter(
                    ['  Unspecified Expansion', 'VTES']),
                    self.aExpectedCards,
                    [None,'VTES']),
        ]

        for tTest in aExpansionTests:
            oFilter, aExpectedCards = self._physical_test(tTest)
            aCards = sorted(oFilter.select(PhysicalCard).distinct())
            self.assertEqual(aCards, aExpectedCards, "Filter Object %s"
                    " failed. %s != %s." % (oFilter, aCards, aExpectedCards))

    def test_card_set_filters(self):
        """Tests for the physical card set filters."""
        # Although splitting this off does add an additional init
        # pass, the logical grouping is fairly different
        aCardSets = [('Test 1', 'Author A', 'A set', False),
                ('Test 2', 'Author B', 'Another set', False),
                ('Test 3', 'Author A', 'Something different', True)]
        aPCSCards = [
                # Set 1
                [('Abombwe', None), ('Alexandra', 'CE'),
                ('Sha-Ennu', None), ('Sha-Ennu', None), ('Sha-Ennu', None),
                ('Sha-Ennu', 'Third Edition')],
                # Set 2
                [('Sha-Ennu', 'Third Edition'), ('Anson', 'Jyhad'),
                    ('.44 magnum', 'Jyhad'), ('ak-47', 'LotN'),
                    ('Alexandra', 'CE'), ('Alexandra', 'CE')],
                # Set 3
                [('Yvette, The Hopeless', 'BSC')]]
        aPCSs = []
        # pylint: disable-msg=E1101
        # sqlobject confuses pylint
        for iCnt, tData in enumerate(aCardSets):
            sName, sAuthor, sComment, bInUse = tData
            oPCS = PhysicalCardSet(name=sName, comment=sComment,
                    author=sAuthor, inuse=bInUse)
            for sName, sExp in aPCSCards[iCnt]:
                if sExp:
                    oExp = IExpansion(sExp)
                else:
                    oExp = None
                oAbs = IAbstractCard(sName)
                oPhys = IPhysicalCard((oAbs, oExp))
                oPCS.addPhysicalCard(oPhys.id)
            aPCSs.append(oPCS)
        # Tests on the physical card set properties
        aPhysicalCardSetTests = [
                (Filters.CardSetNameFilter('Test 1'), [aPCSs[0]]),
                (Filters.CardSetNameFilter('Test'), sorted(aPCSs)),
                (Filters.CSPhysicalCardSetInUseFilter(), [aPCSs[2]]),
                (Filters.CardSetAuthorFilter('Author A'),
                    sorted([aPCSs[0], aPCSs[2]])),
                (Filters.CardSetDescriptionFilter('set'),
                    sorted([aPCSs[0], aPCSs[1]])),
                (Filters.CardSetDescriptionFilter('different'),
                    [aPCSs[2]]),
                ]

        for oFilter, aExpectedSets in aPhysicalCardSetTests:
            aCardSets = sorted(oFilter.select(PhysicalCardSet).distinct())
            self.assertEqual(aCardSets, aExpectedSets, "Filter Object %s"
                    " failed. %s != %s." % (oFilter, aCardSets, aExpectedSets))

        # Test data for the Specific card filters
        oAbsAK = IAbstractCard('ak-47')
        oExp = IExpansion('LotN')
        oPhysAK = IPhysicalCard((oAbsAK, oExp))

        aPCSAbsCardTests = [
                (Filters.PhysicalCardSetFilter('Test 1'),
                    Filters.CardTypeFilter('Vampire'),
                    [u"Alexandra", u"Sha-Ennu", u"Sha-Ennu", u"Sha-Ennu",
                        u"Sha-Ennu"]),
                (Filters.PhysicalCardSetFilter('Test 1'),
                    Filters.CardTypeFilter('Master'),
                    [u"Abombwe"]),
                (Filters.PhysicalCardSetFilter('Test 2'),
                    Filters.CardTypeFilter('Equipment'),
                    [u".44 Magnum", u"AK-47"]),
                (Filters.PhysicalCardSetFilter('Test 2'),
                    Filters.SpecificCardFilter('AK-47'),
                    [u"AK-47"]),
                (Filters.PhysicalCardSetFilter('Test 2'),
                    Filters.SpecificCardIdFilter(oAbsAK.id),
                    [u"AK-47"]),
                (Filters.PhysicalCardSetFilter('Test 2'),
                    Filters.SpecificPhysCardIdFilter(oPhysAK.id),
                    [u"AK-47"]),
                (Filters.PhysicalCardSetFilter('Test 3'),
                    Filters.SpecificPhysCardIdFilter(oPhysAK.id),
                    []),
                ]

        for oPCSFilter, oFilter, aExpectedCards in aPCSAbsCardTests:
            oFullFilter = Filters.FilterAndBox([oPCSFilter, oFilter])
            self.assertTrue('PhysicalCard' in oFullFilter.types)
            aCSCards = [IAbstractCard(x).name for x in oFullFilter.select(
                        MapPhysicalCardToPhysicalCardSet).distinct()]
            self.assertEqual(aCSCards, aExpectedCards, "Filter Object %s"
                    " failed. %s != %s." % (oFullFilter, aCSCards,
                        aExpectedCards))

        aPCSPhysCardTests = [
                (Filters.PhysicalCardSetFilter('Test 1'),
                    Filters.CardTypeFilter('Vampire'),
                    [('Alexandra', 'CE'), ('Sha-Ennu', None),
                        ('Sha-Ennu', None), ('Sha-Ennu', None),
                        ('Sha-Ennu', 'Third Edition')]),
                (Filters.PhysicalCardSetFilter('Test 2'),
                    Filters.SpecificCardFilter('AK-47'),
                    [('AK-47', 'LotN')]),
                (Filters.PhysicalCardSetFilter('Test 3'),
                    Filters.SpecificPhysCardIdFilter(oPhysAK.id),
                    []),
                ]

        for oPCSFilter, oFilter, aExpectedCards in aPCSPhysCardTests:
            oFullFilter = Filters.FilterAndBox([oPCSFilter, oFilter])
            self.assertTrue('PhysicalCard' in oFullFilter.types)
            aCSCards = sorted([IPhysicalCard(x) for x in oFullFilter.select(
                        MapPhysicalCardToPhysicalCardSet).distinct()])
            aExpectedPhysCards = self._convert_to_phys_cards(aExpectedCards)
            self.assertEqual(aCSCards, aExpectedPhysCards, "Filter Object %s"
                    " failed. %s != %s." % (oFullFilter, aCSCards,
                        aExpectedPhysCards))


        aPCSCardsInUse = list(Filters.PhysicalCardSetInUseFilter().select(
                PhysicalCard).distinct())
        aExpectedCards = list(aPCSs[2].cards)
        self.assertEqual(aPCSCardsInUse, aExpectedCards, 'PhysicalCardSet In '
                'Use Filter failed %s != %s' % (aPCSCardsInUse,
                    aExpectedCards))

        # Number tests
        aPCSNumberTests = [
                (Filters.PhysicalCardSetFilter('Test 1'),
                    Filters.CardSetMultiCardCountFilter(('4', aPCSs[0].name)),
                    [u"Sha-Ennu", u"Sha-Ennu", u"Sha-Ennu", u"Sha-Ennu"]),
                (Filters.PhysicalCardSetFilter('Test 1'),
                    Filters.CardSetMultiCardCountFilter(('7', aPCSs[0].name)),
                    []),
                (Filters.PhysicalCardSetFilter('Test 1'),
                    Filters.CardSetMultiCardCountFilter(('1', aPCSs[0].name)),
                    [u"Abombwe", u"Alexandra"]),
                (Filters.PhysicalCardSetFilter('Test 1'),
                    Filters.CardSetMultiCardCountFilter((['1', '4'],
                        aPCSs[0].name)),
                    [u"Abombwe", u"Alexandra", u"Sha-Ennu", u"Sha-Ennu",
                        u"Sha-Ennu", u"Sha-Ennu"]),
                # Cards in 'Test 2' with zero count in 'Test 1'
                (Filters.PhysicalCardSetFilter('Test 2'),
                    Filters.CardSetMultiCardCountFilter(('0', aPCSs[0].name)),
                    [u"Anson", u".44 Magnum", u"AK-47"]),
                    ]

        for oPCSFilter, oFilter, aExpectedCards in aPCSNumberTests:
            # pylint: disable-msg=E1101
            # pyprotocols confuses pylint
            oFullFilter = Filters.FilterAndBox([oPCSFilter, oFilter])
            aCSCards = [IAbstractCard(x).name for x in oFullFilter.select(
                        MapPhysicalCardToPhysicalCardSet).distinct()]
            self.assertEqual(aCSCards, aExpectedCards, "Filter Object %s"
                    " failed. %s != %s." % (oFullFilter, aCSCards,
                        aExpectedCards))

    def test_best_guess_filter(self):
        """Test the best guess filter"""
        # This seems the best fit, to include it with the other filter tests
        aTests = [('Sha-Ennu', [u'Sha-Ennu']),
                ('She-Ennu', [u'Sha-Ennu']),
                ('Saa-Ennu', [u'Sha-Ennu']),
                ('47', [u'AK-47']),
                ('4', [u'.44 Magnum', u'AK-47', u'Earl "Shaka74" Deams']),
                ('Alex', [u'Alexandra']),
                ('Raven', []),
                ('Path of Blood, The', [u'The Path of Blood']),
                ]
        for sGuess, aExpectedNames in aTests:
            oFilter = best_guess_filter(sGuess)
            aNames = [x.name for x in oFilter.select(AbstractCard)]
            self.assertEqual(aNames, aExpectedNames,
                    '%s != expected %s for guess %s' % (aNames, aExpectedNames,
                        sGuess))

if __name__ == "__main__":
    unittest.main()



# CardSetManagementMenu.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Menu for the CardSet View's
# Copyright 2006 Neil Muller <drnlmuller+sutekh@gmail.com>
# Copyright 2006 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details

"""Menu for the card set list"""

import gtk
from sutekh.gui.PaneMenu import PaneMenu

class CardSetManagementMenu(PaneMenu, object):
    """Card Set List Management menu.

       Allows managing the list of card sets (adding new card sets,
       opening card sets, deleting card sets) and filtering the list.
       """
    # pylint: disable-msg=R0904
    # gtk.Widget, so many public methods
    def __init__(self, oFrame, oWindow, oController):
        super(CardSetManagementMenu, self).__init__(oFrame, oWindow)
        self.__sName = 'Card Set List'
        self.__sSetTypeName = 'Card Set'
        self._oController = oController
        self.__create_actions_menu()
        self.create_edit_menu()
        self.create_filter_menu()

    # pylint: disable-msg=W0201
    # called from __init__, so OK
    def __create_actions_menu(self):
        """Add the Actions Menu"""
        oMenu  = self.create_submenu(self, "_Actions")
        self.create_menu_item('Create New Card Set', oMenu,
                self._oController.create_new_card_set)
        self.create_menu_item('Edit Card Set Properties', oMenu,
                self._oController.edit_card_set_properties)
        self.create_menu_item('Mark/UnMark Card Set as in use', oMenu,
                    self._oController.toggle_in_use_flag)
        self.create_menu_item('Delete selected Card Set', oMenu,
                self._oController.delete_card_set, 'Delete')
        oMenu.add(gtk.SeparatorMenuItem())
        self.add_common_actions(oMenu)

    # pylint: enable-msg=W0201

    # pylint: disable-msg=W0613
    # oWidget required by function signature
    def toggle_apply_filter(self, oWidget):
        """Handle menu toggle events"""
        self._oController.run_filter(oWidget.active)

    def set_active_filter(self, oWidget):
        """Handle the menu activate signal"""
        self._oController.get_filter(self)

    def expand_all(self, oWidget):
        """Expand all the rows in the card set."""
        self._oController.view.expand_all()

    def collapse_all(self, oWidget):
        """Collapse all the rows in the card set."""
        self._oController.view.collapse_all()

    def show_search_dialog(self, oWidget):
        """Show the search dialog"""
        self._oController.view.searchdialog.show_all()



# ArdbInfo.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details

"""Base class for the ARDB writers.

   Provide methods for extracting the relevant cards from the card set
   and such.
   """

from sutekh.core.SutekhObjects import IAbstractCard, IPhysicalCard
from sutekh.SutekhInfo import SutekhInfo


def escape_ardb_name(sName):
    """Rework the card name to match ARDB conventions"""
    if sName.startswith('An '):
        sName = sName[3:] + ', An'
    elif sName.startswith('The '):
        sName = sName[4:] + ', The'
    return sName

def unescape_ardb_name(sCardName):
    """Convert from ARDB conventions to Sutekh Conventions"""
    if sCardName.endswith(', The'):
        sCardName = 'The ' + sCardName[:-5]
    elif sCardName.endswith(', An'):
        sCardName = 'An ' + sCardName[:-4]
    return sCardName

def escape_ardb_expansion_name(oExpansion):
    """Rework the expansion name to match ARDB"""
    sExpName = oExpansion.shortname
    if sExpName == 'Promo':
        sExpName = oExpansion.name
        sExpName = sExpName.replace('-', '')
    elif sExpName == 'BSC':
        # ARDB doesn't support BSC.
        sExpName = 'CE'
    return sExpName

def unescape_ardb_expansion_name(sExpName):
    """Rework ARBD Promo to Sutekh Promo-"""
    if sExpName.startswith('Promo') and not sExpName.startswith('Promo-'):
        sExpName = sExpName.replace('Promo', 'Promo-')
    return sExpName


class ArdbInfo(object):
    """Create a string in ARDB's text format representing a dictionary
       of cards."""

    # Should this be an attribute of VersionTable?
    sDatabaseVersion = 'Sutekh-20080331'

    sVersionString = SutekhInfo.VERSION_STR
    # pylint: disable-msg=W0511
    # this is not a actual TODO item
    # Claim same version as recent ARDB
    sFormatVersion = '-TODO-1.0'
    # pyline: enable-msg=W0511


    # pylint: disable-msg=R0201
    # these need to be available to the descendants
    def _extract_crypt(self, dCards):
        """Extract the crypt cards from the list."""
        dCryptStats = {
                'size' : 0,
                'min' : 75,
                'max' : 0,
                'avg' : 0.0
                }
        dVamps = {}
        for tKey, iCount in dCards.iteritems():
            sName = tKey[1]
            # pylint: disable-msg=E1101
            # IAbstractCard confuses pylint
            oCard = IAbstractCard(sName)
            aTypes = [x.name for x in oCard.cardtype]
            if aTypes[0] == 'Vampire':
                dVamps[tKey] = iCount
                dCryptStats['size'] += iCount
                dCryptStats['avg'] += oCard.capacity*iCount
                if oCard.capacity > dCryptStats['max']:
                    dCryptStats['max'] = oCard.capacity
                if oCard.capacity < dCryptStats['min']:
                    dCryptStats['min'] = oCard.capacity
            if aTypes[0] == 'Imbued':
                dVamps[tKey] = iCount
                dCryptStats['size'] += iCount
                dCryptStats['avg'] += oCard.life*iCount
                if oCard.life > dCryptStats['max']:
                    dCryptStats['max'] = oCard.life
                if oCard.life < dCryptStats['min']:
                    dCryptStats['min'] = oCard.life
        if dCryptStats['size'] > 0:
            dCryptStats['avg'] = round(dCryptStats['avg'] /
                    dCryptStats['size'], 2)
        if dCryptStats['min'] == 75:
            dCryptStats['min'] = 0
        return dVamps, dCryptStats

    def _extract_library(self, dCards):
        """Extract the library cards from the list."""
        iSize = 0
        dLib = {}
        for tKey, iCount in dCards.iteritems():
            iId, sName, sSet = tKey
            # pylint: disable-msg=E1101
            # IAbstractCard confuses pylint
            oCard = IAbstractCard(sName)
            aTypes = sorted([x.name for x in oCard.cardtype])
            if aTypes[0] != 'Vampire' and aTypes[0] != 'Imbued':
                # Looks like it should be the right thing, but may not
                sTypeString = "/".join(aTypes)
                # We want to be able to sort over types easily, so
                # we add them to the keys
                dLib[(iId, sName, sTypeString, sSet)] = iCount
                iSize += iCount
        return (dLib, iSize)

    def _gen_disciplines(self, oCard):
        """Extract the discipline string from the card."""
        if len(oCard.discipline) > 0:
            aDisc = []
            for oDisc in oCard.discipline:
                if oDisc.level == 'superior':
                    aDisc.append(oDisc.discipline.name.upper())
                else:
                    aDisc.append(oDisc.discipline.name)
            aDisc.sort() # May not be needed
            return " ".join(aDisc)
        elif len(oCard.virtue) > 0:
            return " ".join(sorted([x.name for x in oCard.virtue]))
        return ""

    def _get_cards(self, oCardIter):
        """Create the dictionary of cards given the list of cards"""
        # pylint: disable-msg=E1101
        # SQLObject methods confuse pylint
        dDict = {}
        for oCard in oCardIter:
            oPhysCard = IPhysicalCard(oCard)
            oAbsCard = IAbstractCard(oCard)
            sSet = self._get_ardb_exp_name(oPhysCard)
            dDict.setdefault((oAbsCard.id, oAbsCard.name, sSet), 0)
            dDict[(oAbsCard.id, oAbsCard.name, sSet)] += 1
        return dDict

    def _get_ardb_exp_name(self, oPhysCard):
        """Extract the correct ARDB name for the expansion"""
        # pylint: disable-msg=E1101
        # IAbstractCard confuses pylint
        if oPhysCard.expansion:
            oExpansion = oPhysCard.expansion
        else:
            oAbsCard = IAbstractCard(oPhysCard)
            # ARDB doesn't have a concept of 'No expansion', so we
            # need to fake it. We use the first legitimate expansion
            # We sort the list to ensure stable results across databases, etc.
            aExp = sorted([oP.expansion for oP in oAbsCard.rarity],
                    key = lambda x: x.shortname)
            oExpansion = aExp[0]
        sSet = escape_ardb_expansion_name(oExpansion)
        return sSet




# WriteArdbInvXML.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2006 Neil Muller <drnlmuller+sutekh@gmail.com>
# Based off the Anarach Revolt Deck Builder xml support,
# ARDB (c) Francios Gombalt, Christoph Boget, Ville Virta and Vincent Ripoll
# GPL - see COPYING for details

"""Given a list of Abstract Cards in a set, write a XML file compatable with
   the Anarch Revolt Deck Builder's XML inventory format."""

from sutekh.core.SutekhObjects import IAbstractCard
from sutekh.core.ArdbInfo import ArdbInfo, escape_ardb_name
from sutekh.SutekhUtility import pretty_xml
import time
# pylint: disable-msg=E0611, F0401
# xml.etree is a python2.5 thing
try:
    from xml.etree.ElementTree import Element, SubElement, ElementTree, \
            tostring
except ImportError:
    from elementtree.ElementTree import Element, SubElement, ElementTree, \
            tostring
# pylint: enable-msg=E0611, F0401


class WriteArdbInvXML(ArdbInfo):
    """Reformat cardset to elementTree and export it to a ARDB
       compatible XML Inventory file."""

    def gen_tree(self, dCards):
        """Creates the actual XML document into memory."""
        # pylint: disable-msg=R0914
        # Need this many local variables to create proper XML tree
        oRoot = Element('inventory')

        sDateWritten = time.strftime('%Y-%m-%d', time.localtime())
        oRoot.attrib['generator'] = "Sutekh [ %s ]" % self.sVersionString
        oRoot.attrib['formatVersion'] = self.sFormatVersion
        oRoot.attrib['databaseVersion'] = self.sDatabaseVersion
        oDateElem = SubElement(oRoot, 'date')
        oDateElem.text = sDateWritten

        dVamps, dCryptStats = self._extract_crypt(dCards)
        (dLib, iLibSize) = self._extract_library(dCards)

        oCryptElem = SubElement(oRoot, 'crypt', size=str(dCryptStats['size']))
        self.format_vamps(oCryptElem, dVamps)

        oLibElem = SubElement(oRoot, 'library', size=str(iLibSize))
        self.format_library(oLibElem, dLib)

        pretty_xml(oRoot)
        return oRoot

    # pylint: disable-msg=R0201
    # these are mthods for consistency
    def format_vamps(self, oCryptElem, dVamps):
        """Convert the Vampire dictionary into elementtree representation."""
        dCombinedVamps = {}
        # ARDB inventory doesn't seperate cards by set, although it's included
        # in the XML file, so we need to combine things so there's only 1
        # entry per card
        for tKey, iNum in dVamps.iteritems():
            iNum = dVamps[tKey]
            iId, sName, sSet = tKey
            iId = tKey[0]
            sName = tKey[1]
            dCombinedVamps.setdefault(iId, [sName, sSet, 0])
            dCombinedVamps[iId][2] += iNum
        for iId, (sName, sSet, iNum) in sorted(dCombinedVamps.iteritems(),
                key=lambda x: x[1][0]):
            # pylint: disable-msg=E1101
            # IAbstractCard confuses pylint
            oCard = IAbstractCard(sName)
            oCardElem = SubElement(oCryptElem, 'vampire',
                    databaseID=str(iId), have=str(iNum), spare='0', need='0')
            # This won't match the ARDB ID's, unless by chance.
            # It looks like that should not be an issue as ARDB will
            # use the name if the IDs don't match
            # It's unclear to me what values ARDB uses here, but
            # these are fine for the xml2html conversion, and look meaningful
            oAdvElem = SubElement(oCardElem, 'adv')
            oNameElem = SubElement(oCardElem, 'name')
            sName = escape_ardb_name(sName)
            if oCard.level is not None:
                oAdvElem.text = '(Advanced)'
                # This is a bit hackish
                oNameElem.text = sName.replace(' (Advanced)', '')
            else:
                oNameElem.text = sName
            oSetElem = SubElement(oCardElem, 'set')
            oSetElem.text = sSet

    def format_library(self, oLibElem, dLib):
        """Format the dictionary of library cards for the element tree."""
        # pylint: disable-msg=R0914
        # Need this many local variables to create proper XML tree
        dCombinedLib = {}
        for tKey, iNum in dLib.iteritems():
            iNum = dLib[tKey]
            iId = tKey[0]
            sName = tKey[1]
            sSet = tKey[3]
            dCombinedLib.setdefault(iId, [sName, sSet, 0])
            dCombinedLib[iId][2] += iNum
        for iId, (sName, sSet, iNum) in sorted(dCombinedLib.iteritems(),
                key=lambda x: x[1][0]):
            oCardElem = SubElement(oLibElem, 'card', databaseID=str(iId),
                    have=str(iNum), spare='0', need='0')
            oNameElem = SubElement(oCardElem, 'name')
            sName = escape_ardb_name(sName)
            oNameElem.text = sName
            oSetElem = SubElement(oCardElem, 'set')
            oSetElem.text = sSet

    # pylint: enable-msg=R0201

    def write(self, fOut, oCardIter):
        """Takes filename, deck details and a dictionary of cards, of the
           form dCard[(id,name)] = count and writes the file."""
        dCards = self._get_cards(oCardIter)
        oRoot = self.gen_tree(dCards)
        ElementTree(oRoot).write(fOut)

    def gen_xml_string(self, oCardIter):
        """Generate string XML representation"""
        dCards = self._get_cards(oCardIter)
        oRoot = self.gen_tree(dCards)
        return tostring(oRoot)





# WriteJOL.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Writer for the JOL format.

   We use the "Num"x"Name" format for created decks, as being shorter

   Example:

   4xVampire 1
   Vampire 2
   2xVampire 3

   2xLib Name

   """

class WriteJOL(object):
    """Create a string in JOL format representing a card set."""

    # pylint: disable-msg=R0201
    # method for consistency with the other methods

    def _escape(self, sName):
        """Escape the card name to JOL's requirements"""
        sName = sName.replace('(Advanced)', '(advanced)')
        return sName

    def _crypt_or_library(self, oCard):
        """Return 'crypt' or 'library' as required"""
        sType = list(oCard.abstractCard.cardtype)[0].name
        if sType == "Vampire" or sType == "Imbued":
            return "crypt"
        else:
            return "library"

    # pylint: enable-msg=R0201
    def _gen_inv(self, oCardSet):
        """Process the card set, creating the lines as needed"""
        dCards = {'crypt' : {}, 'library' : {}}
        sResult = ""
        for oCard in oCardSet.cards:
            sType = self._crypt_or_library(oCard)
            sName = self._escape(oCard.abstractCard.name)
            dCards[sType].setdefault(sName, 0)
            dCards[sType][sName] += 1
        # Sort the output
        for sType in dCards:
            for sName, iNum in sorted(dCards[sType].items()):
                if iNum > 1:
                    sResult += '%dx%s\n' % (iNum, sName)
                else:
                    sResult += '%s\n' % sName
            if sType == 'crypt':
                sResult += '\n'
        # Assume conversion will be handled by viewers/editor/web browser?
        return sResult

    # pylint: enable-msg=R0201

    def write(self, fOut, oCardSet):
        """Takes file object + card set to write, and writes an JOL deck
           representing the deck"""
        fOut.write(self._gen_inv(oCardSet))



# test_WriteArdbInvXML.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Test Writing a card set to an ARDB inventory XML file"""

from sutekh.tests.TestCore import SutekhTest
from sutekh.tests.core.test_PhysicalCardSet import aCardSetNames, \
        get_phys_cards
from sutekh.core.SutekhObjects import PhysicalCardSet
from sutekh.io.WriteArdbInvXML import WriteArdbInvXML
from sutekh.SutekhInfo import SutekhInfo
import unittest
import time

# pylint: disable-msg=W0511, C0301
# W0511 - this is not a actual TODO item
# C0301 - Ignore line length limits for this string
sExpected = """<inventory databaseVersion="%s" formatVersion="-TODO-1.0" generator="Sutekh [ %s ]">
  <date>%s</date>
  <crypt size="2">
    <vampire databaseID="11" have="1" need="0" spare="0">
      <adv />
      <name>Abebe</name>
      <set>LoB</set>
    </vampire><vampire databaseID="19" have="1" need="0" spare="0">
      <adv>(Advanced)</adv>
      <name>Alan Sovereign</name>
      <set>Promo20051001</set>
    </vampire>
  </crypt><library size="9">
    <card databaseID="1" have="2" need="0" spare="0">
      <name>.44 Magnum</name>
      <set>Jyhad</set>
    </card><card databaseID="2" have="2" need="0" spare="0">
      <name>AK-47</name>
      <set>LotN</set>
    </card><card databaseID="8" have="2" need="0" spare="0">
      <name>Abbot</name>
      <set>Third</set>
    </card><card databaseID="14" have="2" need="0" spare="0">
      <name>Abombwe</name>
      <set>LoB</set>
    </card><card databaseID="37" have="1" need="0" spare="0">
      <name>Path of Blood, The</name>
      <set>LotN</set>
    </card>
  </library>
</inventory>""" % (WriteArdbInvXML.sDatabaseVersion, SutekhInfo.VERSION_STR,
        time.strftime('%Y-%m-%d', time.localtime()))
# pylint: enable-msg=W0511, C0301

class ArdbInvXMLWriterTests(SutekhTest):
    """class for the ARDB inventory XML writer tests"""

    def test_deck_writer(self):
        """Test ARDB XML inventroy writing"""
        # pylint: disable-msg=E1101, R0915, R0914
        # E1101: SQLObject + PyProtocols magic confuses pylint
        # R0915, R0914: Want a long, sequential test case to minimise
        # repeated setups, so it has lots of lines + variables
        aAddedPhysCards = get_phys_cards()
        # We have a physical card list, so create some physical card sets
        oPhysCardSet1 = PhysicalCardSet(name=aCardSetNames[0])
        oPhysCardSet1.comment = 'A test comment'
        oPhysCardSet1.author = 'A test author'

        for oCard in aAddedPhysCards:
            oPhysCardSet1.addPhysicalCard(oCard.id)
            oPhysCardSet1.syncUpdate()

        # Check output

        oWriter = WriteArdbInvXML()
        sTempFileName =  self._create_tmp_file()
        fOut = open(sTempFileName, 'w')
        oWriter.write(fOut, oPhysCardSet1.cards)
        fOut.close()

        fIn = open(sTempFileName, 'rU')
        sData = fIn.read()
        fIn.close()

        self.assertEqual(sData, sExpected)

if __name__ == "__main__":
    unittest.main()



# PhysicalCardMenu.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Menu for the Physical Card View
# Copyright 2005, 2006 Simon Cross <hodgestar@gmail.com>
# Copyright 2006 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Menu for the Physical card collection."""

import gtk
from sutekh.gui.PaneMenu import CardListMenu

class PhysicalCardMenu(CardListMenu):
    """Menu for the Physical card collection.

       Enables actions specific to the physical card collection (export to
       file, etc), filtering and plugins.
       """
    # pylint: disable-msg=R0904
    # gtk.Widget, so many public methods
    def __init__(self, oFrame, oController, oWindow):
        super(PhysicalCardMenu, self).__init__(oFrame, oWindow, oController)
        self.__create_physical_cl_menu()
        self.create_edit_menu()
        self.create_filter_menu()
        self.create_plugins_menu('_Plugins', self._oFrame)

    # pylint: disable-msg=W0201
    # called from __init__, so OK
    def __create_physical_cl_menu(self):
        """Create the Actions menu for the card list."""
        # setup sub menu
        oMenu = self.create_submenu(self, "_Actions")

        # items
        self.create_check_menu_item('Show Card Expansions', oMenu,
                self._oController.toggle_expansion, True)

        self.create_check_menu_item('Show icons for the grouping',
                oMenu, self._oController.toggle_icons, True)

        oMenu.add(gtk.SeparatorMenuItem())
        self.add_common_actions(oMenu)

    def create_edit_menu(self):
        """Create the edit menu and populate it"""
        oMenu = self.create_submenu(self, "_Edit")
        self.create_menu_item('Copy selection', oMenu, self.copy_selection,
                '<Ctrl>c')
        self.add_edit_menu_actions(oMenu)





# test_WriteELDBInventory.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Test Writing a card set to an ELDB Inventory"""

from sutekh.tests.TestCore import SutekhTest
from sutekh.tests.core.test_PhysicalCardSet import aCardSetNames, \
        get_phys_cards
from sutekh.core.SutekhObjects import PhysicalCardSet
from sutekh.io.WriteELDBInventory import WriteELDBInventory
import unittest

sExpected = """"ELDB - Inventory"
"Cedric",0,0,"","Crypt"
"Inez "Nurse216" Villagrande",0,0,"","Crypt"
"Alan Sovereign",0,0,"","Crypt"
"Alan Sovereign (ADV)",0,0,"","Crypt"
"Alexandra",0,0,"","Crypt"
".44 Magnum",1,0,"","Library"
"Ablative Skin",0,0,"","Library"
"Aeron",0,0,"","Crypt"
"Gracis Nostinus",0,0,"","Crypt"
"Abombwe",1,0,"","Library"
"Anson",0,0,"","Crypt"
"Kabede Maru",0,0,"","Crypt"
"Amisa",0,0,"","Crypt"
"Kemintiri (ADV)",0,0,"","Crypt"
"Earl "Shaka74" Deams",0,0,"","Crypt"
"Lazar Dobrescu",0,0,"","Crypt"
"Abdelsobek",0,0,"","Crypt"
"Cesewayo",0,0,"","Crypt"
"Angelica, The Canonicus",0,0,"","Crypt"
"Bronwen",0,0,"","Crypt"
"Alfred Benezri",0,0,"","Crypt"
"Path of Blood, The",0,0,"","Library"
"Abjure",0,0,"","Library"
"Yvette, The Hopeless",0,0,"","Crypt"
"AK-47",1,0,"","Library"
"Aire of Elation",0,0,"","Library"
"Abandoning the Flesh",0,0,"","Library"
"Aaron`s Feeding Razor",0,0,"","Library"
"Aabbt Kindred",0,0,"","Crypt"
"Abebe",1,0,"","Crypt"
"Ambrogino Giovanni",0,0,"","Crypt"
"Aaron Duggan, Cameron`s Toady",0,0,"","Crypt"
"Anna "Dictatrix11" Suljic",0,0,"","Crypt"
"Predator`s Communion",0,0,"","Library"
"Akram",0,0,"","Crypt"
"Anastasz di Zagreb",0,0,"","Crypt"
"Abd al-Rashid",0,0,"","Crypt"
"Abbot",1,0,"","Library"
"Sha-Ennu",0,0,"","Crypt"
"Aaron Bathurst",0,0,"","Crypt"
"""

class ELDBInventoryWriterTests(SutekhTest):
    """class for the ELDB Inventory writer tests"""

    def test_inventory_writer(self):
        """Test ELDB inventory writing"""
        # pylint: disable-msg=E1101, R0915, R0914
        # E1101: SQLObject + PyProtocols magic confuses pylint
        # R0915, R0914: Want a long, sequential test case to minimise
        # repeated setups, so it has lots of lines + variables
        aAddedPhysCards = get_phys_cards()
        # We have a physical card list, so create some physical card sets
        oPhysCardSet1 = PhysicalCardSet(name=aCardSetNames[0])
        oPhysCardSet1.comment = 'A test comment'
        oPhysCardSet1.author = 'A test author'

        for iLoop in range(5):
            oPhysCardSet1.addPhysicalCard(aAddedPhysCards[iLoop].id)
            oPhysCardSet1.syncUpdate()

        # Check output

        oWriter = WriteELDBInventory()
        sTempFileName =  self._create_tmp_file()
        fOut = open(sTempFileName, 'w')
        oWriter.write(fOut, oPhysCardSet1)
        fOut.close()

        fIn = open(sTempFileName, 'rU')
        sData = fIn.read()
        fIn.close()

        self.assertEqual(sorted(sData), sorted(sExpected), "Output differs : "
                "%s vs %s" % (sData, sExpected))

if __name__ == "__main__":
    unittest.main()



# CardSetFromFilter.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2006 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details

"""Converts a filter into a card set"""

import gtk
from sutekh.core.SutekhObjects import PhysicalCardSet, PhysicalCard, \
        IPhysicalCard
from sutekh.gui.PluginManager import CardListPlugin
from sutekh.gui.SutekhDialog import do_complaint_error
from sutekh.gui.CreateCardSetDialog import CreateCardSetDialog
from sutekh.gui.CardSetManagementController import update_card_set

class DeckFromFilter(CardListPlugin):
    """Converts a filter into a Card Set."""

    dTableVersions = { PhysicalCardSet : [2, 3, 4, 5]}
    aModelsSupported = [PhysicalCardSet, PhysicalCard]

    def get_menu_item(self):
        """Register on the 'Filter' Menu"""
        if not self.check_versions() or not self.check_model_type():
            return None
        oGenPCS = gtk.MenuItem("Card Set From Filter")
        oGenPCS.connect("activate", self.activate)
        return ('Filter', oGenPCS)

    # pylint: disable-msg=W0613
    # oWidget required by gtk function signature
    def activate(self, oWidget):
        """Create the dialog.

           Prompt the user for Card Set Properties, and so forth.
           """
        oDlg = CreateCardSetDialog(self.parent)
        oDlg.run()

        self.handle_response(oDlg)

    # pylint: enable-msg=W0613

    def handle_response(self, oDlg):
        """Handle the user response from make_dialog

           call make_pcs_from_filter to create the PCS if needed
           """
        sCSName = oDlg.get_name()
        if sCSName:
            oCardSet = self.make_cs_from_filter(sCSName)
            if oCardSet:
                update_card_set(oCardSet, oDlg, self.parent, None)
                self.open_cs(sCSName)

    def make_cs_from_filter(self, sCSName):
        """Create the actual PCS."""
        # Check CS Doesn't Exist
        # pylint: disable-msg=E1101
        # pylint misses PhysicalCardSet methods
        if PhysicalCardSet.selectBy(name=sCSName).count() != 0:
            do_complaint_error("Card Set %s already exists."
                    % sCSName)
            return None

        # Create PCS
        oCS = PhysicalCardSet(name=sCSName)

        # E1101 is still disabled for this
        for oCard in self.model.get_card_iterator(
                self.model.get_current_filter()):
            oCS.addPhysicalCard(IPhysicalCard(oCard))

        return oCS

# pylint: disable-msg=C0103
# accept plugin name
plugin = DeckFromFilter



# ELDBHTMLParser.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# ELDB HTML Deck Parser
# Copyright 2007 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details

# pylint: disable-msg=C0301
# documentation, so line length ignored
"""Parser for ELDB HTML format.

   Example HTML:

   <TR><TD WIDTH=130>Deck Name:</TD><TD WIDTH=520>Osebo Preconstructed Starter Deck</TD></TR>
   <TR><TD WIDTH=130>Created By:</TD><TD WIDTH=520>L. Scott Johnson</TD></TR>
   <TR><TD WIDTH=130 VALIGN="top">Description:</TD><TD WIDTH=520>The Osebo Preconstructed Starter Deck from Legacies of Blood.</TD></TR>
   <TR><TD COLSPAN=2 WIDTH=650>&nbsp;</TD><TR>
   <TR><TD COLSPAN=2 WIDTH=650 BGCOLOR="#eeeeee">Crypt: (12 cards, Min: 12, Max: 36, Avg: 6.00)</TD></TR>
   <TR><TD COLSPAN=2 WIDTH=650>2&nbsp;&nbsp;<a href="http://www.white-wolf.com/vtes/index.php?line=Checklist_LegaciesOfBlood" class="textLink">Uzoma</a> ... </TD></TR>
   ...
   <TR><TD COLSPAN=2 WIDTH=650>2&nbsp;&nbsp;<a href="http://monger.vekn.org/showcard.html?ID=109" class="textLink">Blood Doll</a></TD></TR>
   """
# pylint: enable-msg=C0301

import HTMLParser
import re

# State Base Classes

class StateError(Exception):
    """Error case in the state true"""
    pass

class State(object):
    """Base class for parser states"""
    def __init__(self, oHolder):
        super(State, self).__init__()
        self._sData = ""
        self._oHolder = oHolder

    def transition(self, sTag, dAttr):
        """Transition from one state to another"""
        raise NotImplementedError

    def data(self, sData):
        """Add data to the state"""
        self._sData += sData

# State Classes

class Collecting(State):
    """Default state - transitions to other states as needed"""
    def transition(self, sTag, dAttr):
        """Transition to CardItem of DeckInfoItem as needed."""
        if sTag == 'td' and dAttr.get('colspan') == '2':
            return CardItem(self._oHolder)
        elif sTag == 'td':
            return DeckInfoItem(self._oHolder)
        else:
            return self

class DeckInfoItem(State):
    """States for the table rows describing the deck."""

    # pylint: disable-msg=W0613
    # dAttr required by function signature
    def transition(self, sTag, dAttr):
        """Transition back to Collecting if needed"""
        if sTag == '/tr':
            aParts = self._sData.split(':', 1)

            if len(aParts) != 2:
                return Collecting(self._oHolder)

            sItem, sText = aParts

            if sItem == "Deck Name":
                self._oHolder.name = sText
            elif sItem == "Created By":
                self._oHolder.author = sText
            elif sItem == "Description":
                self._oHolder.comment = sText

            return Collecting(self._oHolder)
        else:
            return self

class CardItem(State):
    """State for the table rows listing the cards in the deck."""
    _oCountRegex = re.compile(r'^[^0-9]*(?P<cnt>[0-9]+)[^0-9]*')

    def __init__(self, oHolder):
        super(CardItem, self).__init__(oHolder)
        self._iCnt = None

    # pylint: disable-msg=W0613
    # dAttr required by function signature
    def transition(self, sTag, dAttr):
        """Extract card data and add it back to the CardSetHolder if possible,
           and transtion back to Collecting if needed."""
        if sTag == 'a':
            oMatch = self._oCountRegex.match(self._sData)
            if oMatch:
                self._iCnt = int(oMatch.group('cnt'))
            else:
                self._iCnt = 1
            self._sData = ""
            return self
        elif sTag == '/a':
            assert(self._iCnt is not None)
            sName = self._sData.strip()
            sName = sName.replace("`", "'")
            # No expansion info for these
            self._oHolder.add(self._iCnt, sName, None)
            self._iCnt = None
            self._sData = ""
            return Collecting(self._oHolder)
        elif sTag == '/tr':
            return Collecting(self._oHolder)
        else:
            return self

# Parser

class ELDBHTMLParser(HTMLParser.HTMLParser, object):
    """Actual Parser for the ELDB HTML files."""
    def __init__(self, oHolder):
        """Create an ELDBHTMLParser.

           oHolder is a sutekh.core.CardSetHolder.CardSetHolder object
           (or similar).
           """
        self._oHolder = oHolder
        self._oState = Collecting(self._oHolder)
        super(ELDBHTMLParser, self).__init__()

    def reset(self):
        """Reset the parser"""
        super(ELDBHTMLParser, self).reset()
        self._oState = Collecting(self._oHolder)

    # pylint: disable-msg=C0111
    # names are as listed in HTMLParser docs, so no need for docstrings
    def handle_starttag(self, sTag, aAttr):
        self._oState = self._oState.transition(sTag.lower(), dict(aAttr))

    def handle_endtag(self, sTag):
        self._oState = self._oState.transition('/' + sTag.lower(), {})

    def handle_data(self, sData):
        self._oState.data(sData)

    # pylint: disable-msg=C0321
    # these don't need statements
    def handle_charref(self, sName): pass
    def handle_entityref(self, sName): pass



# ConfigFile.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Config File handling object
# Wrapper around ConfigParser with some hooks for Sutekh purposes
# Copyright 2007 Neil Muller <drnlmuller+sutekh@gmail.com>
# License: GPL - See COPYRIGHT file for details

"""Config file handling for sutekh"""

from ConfigParser import RawConfigParser, NoOptionError

class ConfigFileListener(object):
    """Listener object for config changes - inspired by CardListModeListener"""

    def add_filter(self, sKey, sFilter):
        """New Filter added"""
        pass

    def remove_filter(self, sKey, sFilter):
        """Filter removed"""
        pass

    def replace_filter(self, sKey, sOldFilter, sNewFilter):
        """A Filter has been replaced"""
        pass

class ConfigFile(object):
    """Handle the setup and management of the config file.

       Ensure that the needed sections exist, and that sensible
       defaults values are assigned.

       Filters are saved to the config file, and interested objects
       can register as listeners on the config file to respond to
       changes to the filters.
       """

    __sFiltersSection = 'Filters'
    __sPanesSection = 'Open Panes'
    __sPrefsSection = 'GUI Preferences'
    __sPluginsSection = 'Plugin Options'
    __sPaneInfoSection = 'Pane settings'

    def __init__(self, sFileName):
        self.__sFileName = sFileName
        self.__oConfig = RawConfigParser()
        # Use read to parse file if it exists
        self.__oConfig.read(self.__sFileName)
        self.__dListeners = {}

        if not self.__oConfig.has_section(self.__sPrefsSection):
            self.__oConfig.add_section(self.__sPrefsSection)
            # default to saving on exit
            self.set_save_on_exit(True)

        if 'save pane sizes' not in self.__oConfig.options(
                self.__sPrefsSection):
            self.set_save_precise_pos(True)

        if 'show zero count cards' not in self.__oConfig.options(
                self.__sPrefsSection):
            self.set_show_zero_count_cards(False)

        if 'save window size' not in self.__oConfig.options(
                self.__sPrefsSection):
            self.set_save_window_size(True)

        if not self.__oConfig.has_section(self.__sPanesSection):
            self.__oConfig.add_section(self.__sPanesSection)
            # No panes information, so we set 'sensible' defaults
            self.add_frame(1, 'physical_card', 'White Wolf Card List', False,
                    -1)
            self.add_frame(2, 'Card Text', 'Card Text', False, -1)
            self.add_frame(3, 'Card Set List', 'Card Set List', False, -1)
            self.add_frame(4, 'physical_card_set', 'My Collection', False, -1)

        if not self.__oConfig.has_section(self.__sFiltersSection):
            self.__oConfig.add_section(self.__sFiltersSection)

        self.__iNum = len(self.__oConfig.items(self.__sFiltersSection))

        if not self.__oConfig.has_section(self.__sPluginsSection):
            self.__oConfig.add_section(self.__sPluginsSection)

        if not self.__oConfig.has_section(self.__sPaneInfoSection):
            self.__oConfig.add_section(self.__sPaneInfoSection)

    def __str__(self):
        """Debugging aid - print the filename"""
        return "FileName : " + self.__sFileName

    def _get_bool_key(self, sSection, sKey):
        """Test if a boolean key is set or not"""
        return self.__oConfig.get(sSection, sKey) == 'yes'

    def _set_bool_key(self, sSection, sKey, bValue):
        """Set a boolean key"""
        if bValue:
            self.__oConfig.set(sSection, sKey, 'yes')
        else:
            self.__oConfig.set(sSection, sKey, 'no')

    def add_listener(self, oListener):
        """Add a listener to respon to config file changes."""
        self.__dListeners[oListener] = None

    def remove_listener(self, oListener):
        """Remove a listener from the list."""
        del self.__dListeners[oListener]

    def write(self):
        """Write the config file to disk."""
        self.__oConfig.write(open(self.__sFileName, 'w'))

    def pre_save_clear(self):
        """Clear out old saved pos, before saving new stuff"""
        aKeys = self.__oConfig.options(self.__sPanesSection)
        for sKey in aKeys:
            self.__oConfig.remove_option(self.__sPanesSection, sKey)
        aKeys = self.__oConfig.options(self.__sPaneInfoSection)
        for sKey in aKeys:
            self.__oConfig.remove_option(self.__sPaneInfoSection, sKey)

    def get_all_panes(self):
        """Get the all the panes saved in the config file, and their
           positions."""
        aRes = []
        for sKey, sValue in self.__oConfig.items(self.__sPanesSection):
            if not sKey.startswith('pane'):
                # invalid key
                continue
            try:
                iPaneNumber = int(sKey.split(' ')[1])
            except ValueError:
                # invalid key
                continue
            # Type is before 1st colon in the name
            if sValue.find(':') == -1:
                # invalid format, so skip
                continue
            sData, sName = sValue.split(':', 1)
            aData = sData.split('.')
            sType = aData[0]
            sPos = '-1'
            bVertical = False
            if len(aData) > 1:
                if aData[1] == 'V':
                    bVertical = True
                    if len(aData) > 2:
                        sPos = aData[2]
                else:
                    sPos = aData[1]
            try:
                iPos = int(sPos)
            except ValueError:
                iPos = -1
            aRes.append((iPaneNumber, sType, sName, bVertical, iPos))
        aRes.sort() # Numbers denote ordering
        return aRes

    def get_all_pane_info(self):
        """Get the pane info (card mode, etc) saved in the config file"""
        dRes = {}
        for tPaneInfo in self.__oConfig.items(self.__sPaneInfoSection):
            # Format is 'pane number' =
            #        Extra Level Mode:Parent Count Mode:Card Mode:Name
            # We can' use the name as a key, since the key isn't case sensitive
            try:
                sExtraLevelMode, sParentCount, sShowCardMode, sName = \
                        tPaneInfo[1].split(':', 3)
                dRes[sName] = (int(sExtraLevelMode), int(sParentCount),
                        int(sShowCardMode))
            except ValueError:
                # skip this one then
                continue
        return dRes

    def get_save_on_exit(self):
        """Query the 'save on exit' option."""
        return self._get_bool_key(self.__sPrefsSection, 'save on exit')

    def set_save_on_exit(self, bSaveOnExit):
        """Set the 'save on exit' option."""
        self._set_bool_key(self.__sPrefsSection, 'save on exit', bSaveOnExit)

    def get_save_precise_pos(self):
        """Query the 'save pane sizes' option."""
        return self._get_bool_key(self.__sPrefsSection, 'save pane sizes')

    def set_save_precise_pos(self, bSavePos):
        """Set the 'save pane sizes' option."""
        self._set_bool_key(self.__sPrefsSection, 'save pane sizes', bSavePos)

    def get_save_window_size(self):
        """Query the 'save window size' option."""
        return self._get_bool_key(self.__sPrefsSection, 'save window size')

    def set_save_window_size(self, bSavePos):
        """Set the 'save window size' option."""
        self._set_bool_key(self.__sPrefsSection, 'save window size', bSavePos)

    def get_show_zero_count_cards(self):
        """Query the 'show zero count cards' option."""
        return self._get_bool_key(self.__sPrefsSection,
                'show zero count cards')

    def set_show_zero_count_cards(self, bShowZero):
        """Set the 'show zero count cards' option."""
        self._set_bool_key(self.__sPrefsSection, 'show zero count cards',
                bShowZero)

    # pylint: disable-msg=R0913
    # We need all the info in the arguments here
    def add_frame(self, iFrameNumber, sType, sName, bVertical, iPos):
        """Add a frame with the given position info to the config file"""
        sKey = 'pane %d' % iFrameNumber
        sData = sType
        if bVertical:
            sData += '.V'
        if iPos > 0 and self.get_save_precise_pos():
            sData += '.%d' % iPos
        sValue = '%s:%s' % (sData, sName)
        self.__oConfig.set(self.__sPanesSection, sKey, sValue)

    # pylint: enable-msg=R0913

    def add_pane_info(self, iFrameNumber, sName, tInfo):
        """Save the pane info"""
        sKey = 'pane %d' % iFrameNumber
        iExtraLevelMode, iParentCount, iShowCardMode = tInfo
        sValue = '%d:%d:%d:%s' % (iExtraLevelMode, iParentCount, iShowCardMode,
                sName)
        self.__oConfig.set(self.__sPaneInfoSection, sKey, sValue)

    def set_database_uri(self, sDatabaseURI):
        """Set the configured database URI"""
        self.__oConfig.set(self.__sPrefsSection, "database url", sDatabaseURI)

    def get_database_uri(self):
        """Get database URI from the config file"""
        try:
            sResult = self.__oConfig.get(self.__sPrefsSection, "database url")
        except NoOptionError:
            sResult = None
        return sResult

    def get_icon_path(self):
        """Get the icon path from the config file"""
        try:
            sResult = self.__oConfig.get(self.__sPrefsSection, "icon path")
        except NoOptionError:
            sResult = None
        return sResult

    def set_icon_path(self, sPath):
        """Set the configured icon path"""
        self.__oConfig.set(self.__sPrefsSection, "icon path", sPath)

    def get_window_size(self):
        """Get the saved window size from the config file."""
        try:
            sResult = self.__oConfig.get(self.__sPrefsSection, 'window size')
            iWidth, iHeight = [int(x) for x in sResult.split(',')]
        except NoOptionError:
            iWidth, iHeight = -1, -1
        return iWidth, iHeight

    def save_window_size(self, tSize):
        """Save the current window size."""
        sPos = '%d, %d' % (tSize[0], tSize[1])
        self.__oConfig.set(self.__sPrefsSection, 'window size', sPos)

    # filters section

    def get_filters(self):
        """Get all the filters in the config file."""
        return [x[1] for x in self.__oConfig.items(self.__sFiltersSection)]

    def get_filter_keys(self):
        """Get the keys used to reference the filters."""
        return self.__oConfig.items(self.__sFiltersSection)

    def get_filter(self, sKey):
        """Return the filter associated with the given key or None."""
        if sKey.lower() in self.__oConfig.options(self.__sFiltersSection):
            return self.__oConfig.get(self.__sFiltersSection, sKey)
        else:
            return None

    def add_filter(self, sKey, sFilter):
        """Add a filter to the config file."""
        if sKey.lower() not in self.__oConfig.options(self.__sFiltersSection):
            self.__oConfig.set(self.__sFiltersSection, sKey, sFilter)
            for oListener in self.__dListeners:
                oListener.add_filter(sKey, sFilter)

    def remove_filter(self, sKey, sFilter):
        """Remove a filter from the file"""
        # Make sure Filer and key match
        if sKey.lower() in self.__oConfig.options(self.__sFiltersSection) and \
                sFilter == self.__oConfig.get(self.__sFiltersSection, sKey):
            self.__oConfig.remove_option(self.__sFiltersSection, sKey)
            for oListener in self.__dListeners:
                oListener.remove_filter(sKey, sFilter)

    def replace_filter(self, sKey, sOldFilter, sNewFilter):
        """Replace a filter in the file with new filter"""
        if sKey.lower() in self.__oConfig.options(self.__sFiltersSection) and \
                sOldFilter == self.__oConfig.get(self.__sFiltersSection, sKey):
            self.__oConfig.remove_option(self.__sFiltersSection, sKey)
            self.__oConfig.set(self.__sFiltersSection, sKey, sNewFilter)
            for oListener in self.__dListeners:
                oListener.replace_filter(sKey, sOldFilter, sNewFilter)

    # plugins section

    def get_plugin_key(self, sKey):
        """Get an option from the plugins section.

           Return None if no option is set
           """
        try:
            sResult = self.__oConfig.get(self.__sPluginsSection, sKey)
        except NoOptionError:
            sResult = None
        return sResult

    def set_plugin_key(self, sKey, sValue):
        """Set a value in the plugin section"""
        self.__oConfig.set(self.__sPluginsSection, sKey, sValue)



# setup.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2007 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details

# Be sure that the import will grab the correct version
# of SutekhInfo when building packages.

"""Setuptools setup.py file for Sutekh."""

from setuptools import setup, find_packages
from sutekh.SutekhInfo import SutekhInfo

setup   (   # Metadata
            name = SutekhInfo.NAME,
            version = SutekhInfo.VERSION_STR,
            description = SutekhInfo.DESCRIPTION,

            author = SutekhInfo.AUTHOR_NAME,
            author_email = SutekhInfo.AUTHOR_EMAIL,

            maintainer = SutekhInfo.MAINTAINER_NAME,
            maintainer_email = SutekhInfo.MAINTAINER_EMAIL,

            url = SutekhInfo.SOURCEFORGE_URL,
            download_url = SutekhInfo.PYPI_URL,

            license = SutekhInfo.LICENSE,

            classifiers = SutekhInfo.CLASSIFIERS,

            platforms = SutekhInfo.PLATFORMS,

            # Dependencies
            install_requires = SutekhInfo.INSTALL_REQUIRES,

            # Files
            packages = find_packages(exclude=['sutekh.tests']),
            package_data = {
                # Include SVG files from all packages
                '': ['*.svg'],
                # Include LICENSE information for sutekh package
                # Include everything under the docs directory
                'sutekh': ['COPYING', 'docs/html/*'],
            },
            scripts = ['sutekh/SutekhCli.py','sutekh/SutekhGui.py']
        )
